@@ -1,24 +1,26 @@
-/* file locking
+/*
+   File locking
 
-   Copyright (C) 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2011
+   The Free Software Foundation, Inc.
 
-   Authors: 2003 Adam Byrtek
+   Written by:
+   Adam Byrtek, 2003
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
-
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /** \file
@@ -53,7 +55,7 @@
 #include <stdlib.h>
 
 #include "lib/global.h"
-#include "lib/vfs/mc-vfs/vfs.h"
+#include "lib/vfs/vfs.h"
 #include "lib/util.h"           /* tilde_expand() */
 #include "lib/lock.h"
 #include "lib/widget.h"         /* query_dialog() */
@@ -197,6 +199,7 @@ lock_file (const char *fname)
     struct stat statbuf;
     struct lock_s *lockinfo;
     gboolean symlink_ok;
+    vfs_path_t *vpath;
 
     /* Just to be sure (and don't lock new file) */
     if (fname == NULL || *fname == '\0')
@@ -204,12 +207,16 @@ lock_file (const char *fname)
 
     fname = tilde_expand (fname);
 
+    vpath = vfs_path_from_str (fname);
+
     /* Locking on VFS is not supported */
-    if (!vfs_file_is_local (fname))
+    if (!vfs_file_is_local (vpath))
     {
         g_free ((gpointer) fname);
+        vfs_path_free (vpath);
         return 0;
     }
+    vfs_path_free (vpath);
 
     /* Check if already locked */
     lockfname = lock_build_symlink_name (fname);
