@@ -1,27 +1,27 @@
 #ifndef MC_CONFIG_H
 #define MC_CONFIG_H
 
-/*** typedefs(not structures) and defined constants ********************/
+/*** typedefs(not structures) and defined constants **********************************************/
 
 #define CONFIG_APP_SECTION "Midnight-Commander"
 #define CONFIG_PANELS_SECTION "Panels"
 
-/*** enums *************************************************************/
+/*** enums ***************************************************************************************/
 
-/*** structures declarations (and typedefs of structures)***************/
+/*** structures declarations (and typedefs of structures)*****************************************/
 
-typedef struct mc_config_struct
+typedef struct mc_config_t
 {
     GKeyFile *handle;
     gchar *ini_path;
 } mc_config_t;
 
-/*** global variables defined in .c file *******************************/
+/*** global variables defined in .c file *********************************************************/
 
 extern mc_config_t *mc_main_config;
 extern mc_config_t *mc_panels_config;
 
-/*** declarations of public functions **********************************/
+/*** declarations of public functions ************************************************************/
 
 /* mcconfig/common.c: */
 
@@ -31,10 +31,11 @@ void mc_config_deinit (mc_config_t *);
 gboolean mc_config_del_key (mc_config_t *, const char *, const gchar *);
 gboolean mc_config_del_group (mc_config_t *, const char *);
 
-gboolean mc_config_has_param (mc_config_t *, const char *, const gchar *);
+gboolean mc_config_has_param (const mc_config_t *, const char *, const gchar *);
 gboolean mc_config_has_group (mc_config_t *, const char *);
 
-gboolean mc_config_read_file (mc_config_t *, const gchar *);
+gboolean mc_config_read_file (mc_config_t * mc_config, const gchar * ini_path,
+                              gboolean remove_empty);
 
 gboolean mc_config_save_file (mc_config_t * config, GError ** error);
 
@@ -43,13 +44,13 @@ gboolean mc_config_save_to_file (mc_config_t * config, const gchar * filename, G
 
 /* mcconfig/get.c: */
 
-gchar **mc_config_get_groups (mc_config_t *, gsize *);
+gchar **mc_config_get_groups (const mc_config_t *, gsize *);
 
-gchar **mc_config_get_keys (mc_config_t *, const gchar *, gsize *);
+gchar **mc_config_get_keys (const mc_config_t *, const gchar *, gsize *);
 
 gchar *mc_config_get_string (mc_config_t *, const gchar *, const gchar *, const gchar *);
 
-gchar *mc_config_get_string_raw (mc_config_t *, const gchar *, const gchar *, const gchar *);
+gchar *mc_config_get_string_raw (const mc_config_t *, const gchar *, const gchar *, const gchar *);
 
 gboolean mc_config_get_bool (mc_config_t *, const gchar *, const gchar *, gboolean);
 
@@ -69,7 +70,7 @@ void mc_config_set_string_raw (mc_config_t *, const gchar *, const gchar *, cons
 
 void mc_config_set_string_raw_value (mc_config_t *, const gchar *, const gchar *, const gchar *);
 
-void mc_config_set_string (mc_config_t *, const gchar *, const gchar *, const gchar *);
+void mc_config_set_string (const mc_config_t *, const gchar *, const gchar *, const gchar *);
 
 void mc_config_set_bool (mc_config_t *, const gchar *, const gchar *, gboolean);
 
@@ -88,4 +89,28 @@ void mc_config_set_int_list (mc_config_t *, const gchar *, const gchar *, int[],
 
 void mc_config_show_dialog (void);
 
+
+/* mcconfig/paths.c: */
+
+void mc_config_init_config_paths (GError ** error);
+
+void mc_config_deinit_config_paths (void);
+
+gboolean mc_config_deprecated_dir_present (void);
+
+void mc_config_migrate_from_old_place (GError ** error);
+
+const char *mc_config_get_data_path (void);
+
+const char *mc_config_get_cache_path (void);
+
+const char *mc_config_get_path (void);
+
+const char *mc_config_get_home_dir (void);
+
+char *mc_config_get_full_path (const char *config_name);
+
+
+/*** inline functions ****************************************************************************/
+
 #endif
