@@ -2,27 +2,27 @@
    Search text engine.
    Regex search
 
-   Copyright (C) 2009 The Free Software Foundation, Inc.
+   Copyright (C) 2009, 2011
+   The Free Software Foundation, Inc.
 
    Written by:
-   Slava Zanko <slavazanko@gmail.com>, 2009.
+   Slava Zanko <slavazanko@gmail.com>, 2009,2010,2011
+   Vitaliy Filippov <vitalif@yourcmc.ru>, 2011
 
    This file is part of the Midnight Commander.
 
-   The Midnight Commander is free software; you can redistribute it
+   The Midnight Commander is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
 
-   The Midnight Commander is distributed in the hope that it will be
-   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   The Midnight Commander is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -42,6 +42,10 @@
 
 /*** file scope macro definitions ****************************************************************/
 
+#define REPLACE_PREPARE_T_NOTHING_SPECIAL -1
+#define REPLACE_PREPARE_T_REPLACE_FLAG    -2
+#define REPLACE_PREPARE_T_ESCAPE_SEQ      -3
+
 /*** file scope type declarations ****************************************************************/
 
 typedef enum
@@ -53,6 +57,7 @@ typedef enum
     REPLACE_T_LOW_TRANSFORM = 8
 } replace_transform_type_t;
 
+
 /*** file scope variables ************************************************************************/
 
 /*** file scope functions ************************************************************************/
@@ -374,6 +379,85 @@ mc_search_regex__get_token_by_num (const mc_search_t * lc_mc_search, gsize lc_in
 
 /* --------------------------------------------------------------------------------------------- */
 
+static gboolean
+mc_search_regex__replace_handle_esc_seq (const GString * replace_str, const gsize current_pos,
+                                         gsize * skip_len, int *ret)
+{
+    char *curr_str = &(replace_str->str[current_pos]);
+    char c = *(curr_str + 1);
+
+    if (replace_str->len > current_pos + 2)
+    {
+        if (c == '{')
+        {
+            for (*skip_len = 2; /* \{ */
+                 current_pos + *skip_len < replace_str->len
+                 && *(curr_str + *skip_len) >= '0'
+                 && *(curr_str + *skip_len) <= '7'; (*skip_len)++);
+            if (current_pos + *skip_len < replace_str->len && *(curr_str + *skip_len) == '}')
+            {
+                (*skip_len)++;
+                *ret = REPLACE_PREPARE_T_ESCAPE_SEQ;
+                return FALSE;
+            }
+            else
+            {
+                *ret = REPLACE_PREPARE_T_NOTHING_SPECIAL;
+                return TRUE;
+            }
+        }
+
+        if (c == 'x')
+        {
+            *skip_len = 2;      /* \x */
+            c = *(curr_str + 2);
+            if (c == '{')
+            {
+                for (*skip_len = 3;     /* \x{ */
+                     current_pos + *skip_len < replace_str->len
+                     && g_ascii_isxdigit ((guchar) * (curr_str + *skip_len)); (*skip_len)++);
+                if (current_pos + *skip_len < replace_str->len && *(curr_str + *skip_len) == '}')
+                {
+                    (*skip_len)++;
+                    *ret = REPLACE_PREPARE_T_ESCAPE_SEQ;
+                    return FALSE;
+                }
+                else
+                {
+                    *ret = REPLACE_PREPARE_T_NOTHING_SPECIAL;
+                    return TRUE;
+                }
+            }
+            else if (!g_ascii_isxdigit ((guchar) c))
+            {
+                *skip_len = 2;  /* \x without number behind */
+                *ret = REPLACE_PREPARE_T_NOTHING_SPECIAL;
+                return FALSE;
+            }
+            else
+            {
+                c = *(curr_str + 3);
+                if (!g_ascii_isxdigit ((guchar) c))
+                    *skip_len = 3;      /* \xH */
+                else
+                    *skip_len = 4;      /* \xHH */
+                *ret = REPLACE_PREPARE_T_ESCAPE_SEQ;
+                return FALSE;
+            }
+        }
+    }
+
+    if (strchr ("ntvbrfa", c) != NULL)
+    {
+        *skip_len = 2;
+        *ret = REPLACE_PREPARE_T_ESCAPE_SEQ;
+        return FALSE;
+    }
+    return TRUE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 mc_search_regex__process_replace_str (const GString * replace_str, const gsize current_pos,
                                       gsize * skip_len, replace_transform_type_t * replace_flags)
@@ -383,16 +467,17 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
     const char *curr_str = &(replace_str->str[current_pos]);
 
     if (current_pos > replace_str->len)
-        return -1;
+        return REPLACE_PREPARE_T_NOTHING_SPECIAL;
 
     *skip_len = 0;
 
-    if (*curr_str == '$' && *(curr_str + 1) == '{' && (*(curr_str + 2) & (char) 0xf0) == 0x30)
+    if ((*curr_str == '$') && (*(curr_str + 1) == '{') && ((*(curr_str + 2) & (char) 0xf0) == 0x30)
+        && (replace_str->len > current_pos + 2))
     {
         if (strutils_is_char_escaped (replace_str->str, curr_str))
         {
             *skip_len = 1;
-            return -1;
+            return REPLACE_PREPARE_T_NOTHING_SPECIAL;
         }
 
         for (*skip_len = 0;
@@ -400,34 +485,38 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
              && (*(curr_str + 2 + *skip_len) & (char) 0xf0) == 0x30; (*skip_len)++);
 
         if (*(curr_str + 2 + *skip_len) != '}')
-            return -1;
+            return REPLACE_PREPARE_T_NOTHING_SPECIAL;
 
         tmp_str = g_strndup (curr_str + 2, *skip_len);
         if (tmp_str == NULL)
-            return -1;
+            return REPLACE_PREPARE_T_NOTHING_SPECIAL;
 
         ret = atoi (tmp_str);
         g_free (tmp_str);
 
         *skip_len += 3;         /* ${} */
-        return ret;
+        return ret;             /* capture buffer index >= 0 */
     }
 
-    if (*curr_str == '\\')
+    if ((*curr_str == '\\') && (replace_str->len > current_pos + 1))
     {
         if (strutils_is_char_escaped (replace_str->str, curr_str))
         {
             *skip_len = 1;
-            return -1;
+            return REPLACE_PREPARE_T_NOTHING_SPECIAL;
         }
 
         if (g_ascii_isdigit (*(curr_str + 1)))
         {
-            ret = g_ascii_digit_value (*(curr_str + 1));
+            ret = g_ascii_digit_value (*(curr_str + 1));        /* capture buffer index >= 0 */
             *skip_len = 2;      /* \\ and one digit */
             return ret;
         }
-        ret = -2;
+
+        if (!mc_search_regex__replace_handle_esc_seq (replace_str, current_pos, skip_len, &ret))
+            return ret;
+
+        ret = REPLACE_PREPARE_T_REPLACE_FLAG;
         *skip_len += 2;
         switch (*(curr_str + 1))
         {
@@ -449,13 +538,15 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
             *replace_flags = REPLACE_T_NO_TRANSFORM;
             break;
         default:
-            ret = -1;
+            ret = REPLACE_PREPARE_T_NOTHING_SPECIAL;
             break;
         }
     }
     return ret;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 mc_search_regex__process_append_str (GString * dest_str, const char *from, gsize len,
                                      replace_transform_type_t * replace_flags)
@@ -517,7 +608,111 @@ mc_search_regex__process_append_str (GString * dest_str, const char *from, gsize
 
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+mc_search_regex__process_escape_sequence (GString * dest_str, const char *from, gsize len,
+                                          replace_transform_type_t * replace_flags,
+                                          gboolean is_utf8)
+{
+    gsize i = 0;
+    unsigned int c = 0;
+    char b;
+
+    if (len == (gsize) (-1))
+        len = strlen (from);
+    if (len == 0)
+        return;
+    if (from[i] == '{')
+        i++;
+    if (i >= len)
+        return;
+
+    if (from[i] == 'x')
+    {
+        i++;
+        if (i < len && from[i] == '{')
+            i++;
+        for (; i < len; i++)
+        {
+            if (from[i] >= '0' && from[i] <= '9')
+                c = c * 16 + from[i] - '0';
+            else if (from[i] >= 'a' && from[i] <= 'f')
+                c = c * 16 + 10 + from[i] - 'a';
+            else if (from[i] >= 'A' && from[i] <= 'F')
+                c = c * 16 + 10 + from[i] - 'A';
+            else
+                break;
+        }
+    }
+    else if (from[i] >= '0' && from[i] <= '7')
+        for (; i < len && from[i] >= '0' && from[i] <= '7'; i++)
+            c = c * 8 + from[i] - '0';
+    else
+    {
+        switch (from[i])
+        {
+        case 'n':
+            c = '\n';
+            break;
+        case 't':
+            c = '\t';
+            break;
+        case 'v':
+            c = '\v';
+            break;
+        case 'b':
+            c = '\b';
+            break;
+        case 'r':
+            c = '\r';
+            break;
+        case 'f':
+            c = '\f';
+            break;
+        case 'a':
+            c = '\a';
+            break;
+        default:
+            mc_search_regex__process_append_str (dest_str, from, len, replace_flags);
+            return;
+        }
+    }
+
+    if (c < 0x80 || !is_utf8)
+        g_string_append_c (dest_str, (char) c);
+    else if (c < 0x800)
+    {
+        b = 0xC0 | (c >> 6);
+        g_string_append_c (dest_str, b);
+        b = 0x80 | (c & 0x3F);
+        g_string_append_c (dest_str, b);
+    }
+    else if (c < 0x10000)
+    {
+        b = 0xE0 | (c >> 12);
+        g_string_append_c (dest_str, b);
+        b = 0x80 | ((c >> 6) & 0x3F);
+        g_string_append_c (dest_str, b);
+        b = 0x80 | (c & 0x3F);
+        g_string_append_c (dest_str, b);
+    }
+    else if (c < 0x10FFFF)
+    {
+        b = 0xF0 | (c >> 16);
+        g_string_append_c (dest_str, b);
+        b = 0x80 | ((c >> 12) & 0x3F);
+        g_string_append_c (dest_str, b);
+        b = 0x80 | ((c >> 6) & 0x3F);
+        g_string_append_c (dest_str, b);
+        b = 0x80 | (c & 0x3F);
+        g_string_append_c (dest_str, b);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
 
 void
 mc_search__cond_struct_new_init_regex (const char *charset, mc_search_t * lc_mc_search,
@@ -525,10 +720,6 @@ mc_search__cond_struct_new_init_regex (const char *charset, mc_search_t * lc_mc_
 {
 #ifdef SEARCH_TYPE_GLIB
     GError *error = NULL;
-#else /* SEARCH_TYPE_GLIB */
-    const char *error;
-    int erroffset;
-#endif /* SEARCH_TYPE_GLIB */
 
     if (!lc_mc_search->is_case_sensitive)
     {
@@ -538,10 +729,9 @@ mc_search__cond_struct_new_init_regex (const char *charset, mc_search_t * lc_mc_
         mc_search_cond->str = mc_search__cond_struct_new_regex_ci_str (charset, tmp);
         g_string_free (tmp, TRUE);
     }
-#ifdef SEARCH_TYPE_GLIB
     mc_search_cond->regex_handle =
-        g_regex_new (mc_search_cond->str->str, G_REGEX_OPTIMIZE | G_REGEX_RAW | G_REGEX_DOTALL, 0,
-                     &error);
+        g_regex_new (mc_search_cond->str->str, G_REGEX_OPTIMIZE | G_REGEX_RAW | G_REGEX_DOTALL,
+                     0, &error);
 
     if (error != NULL)
     {
@@ -551,8 +741,30 @@ mc_search__cond_struct_new_init_regex (const char *charset, mc_search_t * lc_mc_
         return;
     }
 #else /* SEARCH_TYPE_GLIB */
+    const char *error;
+    int erroffset;
+    int pcre_options = PCRE_EXTRA | PCRE_MULTILINE;
+
+    if (str_isutf8 (charset) && mc_global.utf8_display)
+    {
+        pcre_options |= PCRE_UTF8;
+        if (lc_mc_search->is_case_sensitive)
+            pcre_options |= PCRE_CASELESS;
+    }
+    else
+    {
+        if (!lc_mc_search->is_case_sensitive)
+        {
+            GString *tmp;
+
+            tmp = mc_search_cond->str;
+            mc_search_cond->str = mc_search__cond_struct_new_regex_ci_str (charset, tmp);
+            g_string_free (tmp, TRUE);
+        }
+    }
+
     mc_search_cond->regex_handle =
-        pcre_compile (mc_search_cond->str->str, PCRE_EXTRA, &error, &erroffset, NULL);
+        pcre_compile (mc_search_cond->str->str, pcre_options, &error, &erroffset, NULL);
     if (mc_search_cond->regex_handle == NULL)
     {
         lc_mc_search->error = MC_SEARCH_E_REGEX_COMPILE;
@@ -572,6 +784,7 @@ mc_search__cond_struct_new_init_regex (const char *charset, mc_search_t * lc_mc_
         }
     }
 #endif /* SEARCH_TYPE_GLIB */
+    lc_mc_search->is_utf8 = str_isutf8 (charset);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -711,7 +924,7 @@ mc_search_regex_prepare_replace_str (mc_search_t * lc_mc_search, GString * repla
     {
         lc_index = mc_search_regex__process_replace_str (replace_str, loop, &len, &replace_flags);
 
-        if (lc_index == -1)
+        if (lc_index == REPLACE_PREPARE_T_NOTHING_SPECIAL)
         {
             if (len != 0)
             {
@@ -726,7 +939,7 @@ mc_search_regex_prepare_replace_str (mc_search_t * lc_mc_search, GString * repla
             continue;
         }
 
-        if (lc_index == -2)
+        if (lc_index == REPLACE_PREPARE_T_REPLACE_FLAG)
         {
             if (loop)
                 mc_search_regex__process_append_str (ret, prev_str,
@@ -737,6 +950,21 @@ mc_search_regex_prepare_replace_str (mc_search_t * lc_mc_search, GString * repla
             continue;
         }
 
+        /* escape sequence */
+        if (lc_index == REPLACE_PREPARE_T_ESCAPE_SEQ)
+        {
+            mc_search_regex__process_append_str (ret, prev_str,
+                                                 replace_str->str + loop - prev_str,
+                                                 &replace_flags);
+            /* call process_escape_sequence without starting '\\' */
+            mc_search_regex__process_escape_sequence (ret, replace_str->str + loop + 1, len - 1,
+                                                      &replace_flags, lc_mc_search->is_utf8);
+            prev_str = replace_str->str + loop + len;
+            loop += len - 1;
+            continue;
+        }
+
+        /* invalid capture buffer number */
         if (lc_index > lc_mc_search->num_results)
         {
             g_string_free (ret, TRUE);
