@@ -1,23 +1,28 @@
-/* Mouse managing
+/*
+   Mouse managing
+
    Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2004, 2005, 2006,
-   2007, 2009 Free Software Foundation, Inc.
+   2007, 2009, 2011
+   The Free Software Foundation, Inc.
 
    Written by:
    Andrew Borodin <aborodin@vmail.ru>, 2009.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** \file mouse.c
  *  \brief Source: mouse managing
@@ -43,7 +48,6 @@
 Mouse_Type use_mouse_p = MOUSE_NONE;
 gboolean mouse_enabled = FALSE;
 const char *xmouse_seq;
-gboolean old_mouse = FALSE;
 
 /*** file scope macro definitions ****************************************************************/
 
@@ -134,6 +138,9 @@ enable_mouse (void)
         /* enable mouse tracking */
         printf (ESC_STR "[?1000h");
 
+        /* enable urxvt extended mouse coordinate reporting */
+        printf (ESC_STR "[?1015h");
+
         fflush (stdout);
         mouse_enabled = TRUE;
         break;
@@ -145,6 +152,9 @@ enable_mouse (void)
         /* enable mouse tracking */
         printf (ESC_STR "[?1002h");
 
+        /* enable urxvt extended mouse coordinate reporting */
+        printf (ESC_STR "[?1015h");
+
         fflush (stdout);
         mouse_enabled = TRUE;
         break;
@@ -172,6 +182,9 @@ disable_mouse (void)
         break;
 #endif
     case MOUSE_XTERM_NORMAL_TRACKING:
+        /* disable urxvt extended mouse coordinate reporting */
+        printf (ESC_STR "[?1015l");
+
         /* disable mouse tracking */
         printf (ESC_STR "[?1000l");
 
@@ -181,6 +194,9 @@ disable_mouse (void)
         fflush (stdout);
         break;
     case MOUSE_XTERM_BUTTON_EVENT_TRACKING:
+        /* disable urxvt extended mouse coordinate reporting */
+        printf (ESC_STR "[?1015l");
+
         /* disable mouse tracking */
         printf (ESC_STR "[?1002l");
 
