@@ -1,24 +1,29 @@
 /* {{{ Copyright */
 
 /* Background support.
-   Copyright (C) 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007
-   Free Software Foundation, Inc.
 
-   Written by: 1996 Miguel de Icaza
+   Copyright (C) 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007,
+   2011
+   The Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   Written by:
+   Miguel de Icaza, 1996
 
-   This program is distributed in the hope that it will be useful,
+   This file is part of the Midnight Commander.
+
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /* }}} */
 
@@ -46,8 +51,8 @@
 #include "lib/global.h"
 #include "lib/tty/key.h"        /* add_select_channel(), delete_select_channel() */
 #include "lib/widget.h"         /* message() */
+#include "lib/event-types.h"
 
-#include "filemanager/layout.h" /* repaint_screen() */
 #include "filemanager/fileopctx.h"      /* FileOpContext */
 
 #include "background.h"
@@ -56,9 +61,6 @@
 
 #define MAXCALLARGS 4           /* Number of arguments supported */
 
-/* If true, this is a background process */
-int we_are_background = 0;
-
 /*** file scope macro definitions ****************************************************************/
 
 /*** file scope type declarations ****************************************************************/
@@ -411,6 +413,71 @@ parent_call_header (void *routine, int argc, enum ReturnType type, FileOpContext
         ret = write (parent_fd, ctx, sizeof (FileOpContext));
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+static int
+parent_va_call (void *routine, gpointer data, int argc, va_list ap)
+{
+    int i;
+    ssize_t ret;
+    struct FileOpContext *ctx = (struct FileOpContext *) data;
+
+    parent_call_header (routine, argc, Return_Integer, ctx);
+    for (i = 0; i < argc; i++)
+    {
+        int len;
+        void *value;
+
+        len = va_arg (ap, int);
+        value = va_arg (ap, void *);
+        ret = write (parent_fd, &len, sizeof (int));
+        ret = write (parent_fd, value, len);
+    }
+
+    ret = read (from_parent_fd, &i, sizeof (int));
+    if (ctx)
+        ret = read (from_parent_fd, ctx, sizeof (FileOpContext));
+
+    return i;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static char *
+parent_va_call_string (void *routine, int argc, va_list ap)
+{
+    char *str;
+    int i;
+
+    parent_call_header (routine, argc, Return_String, NULL);
+    for (i = 0; i < argc; i++)
+    {
+        int len;
+        void *value;
+
+        len = va_arg (ap, int);
+        value = va_arg (ap, void *);
+        if ((write (parent_fd, &len, sizeof (int)) != sizeof (int)) ||
+            (write (parent_fd, value, len) != len))
+        {
+            return NULL;
+        }
+    }
+
+    if (read (from_parent_fd, &i, sizeof (int)) != sizeof (int))
+        return NULL;
+    if (!i)
+        return NULL;
+    str = g_malloc (i + 1);
+    if (read (from_parent_fd, str, i) != i)
+    {
+        g_free (str);
+        return NULL;
+    }
+    str[i] = 0;
+    return str;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -475,7 +542,7 @@ do_background (struct FileOpContext *ctx, char *info)
         parent_fd = comm[1];
         from_parent_fd = back_comm[0];
 
-        we_are_background = 1;
+        mc_global.we_are_background = 1;
         top_dlg = NULL;
 
         /* Make stdin/stdout/stderr point somewhere */
@@ -509,28 +576,14 @@ do_background (struct FileOpContext *ctx, char *info)
 int
 parent_call (void *routine, struct FileOpContext *ctx, int argc, ...)
 {
+    int ret;
     va_list ap;
-    int i;
-    ssize_t ret;
 
     va_start (ap, argc);
-    parent_call_header (routine, argc, Return_Integer, ctx);
-    for (i = 0; i < argc; i++)
-    {
-        int len;
-        void *value;
+    ret = parent_va_call (routine, (gpointer) ctx, argc, ap);
+    va_end (ap);
 
-        len = va_arg (ap, int);
-        value = va_arg (ap, void *);
-        ret = write (parent_fd, &len, sizeof (int));
-        ret = write (parent_fd, value, len);
-    }
-
-    ret = read (from_parent_fd, &i, sizeof (int));
-    if (ctx)
-        ret = read (from_parent_fd, ctx, sizeof (FileOpContext));
-
-    return i;
+    return ret;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -540,35 +593,52 @@ parent_call_string (void *routine, int argc, ...)
 {
     va_list ap;
     char *str;
-    int i;
 
     va_start (ap, argc);
-    parent_call_header (routine, argc, Return_String, NULL);
-    for (i = 0; i < argc; i++)
-    {
-        int len;
-        void *value;
+    str = parent_va_call_string (routine, argc, ap);
+    va_end (ap);
 
-        len = va_arg (ap, int);
-        value = va_arg (ap, void *);
-        if ((write (parent_fd, &len, sizeof (int)) != sizeof (int)) ||
-            (write (parent_fd, value, len) != len))
-            return NULL;
-    }
-    if (read (from_parent_fd, &i, sizeof (int)) != sizeof (int))
-        return NULL;
-    if (!i)
-        return NULL;
-    str = g_malloc (i + 1);
-    if (read (from_parent_fd, str, i) != i)
-    {
-        g_free (str);
-        return NULL;
-    }
-    str[i] = 0;
     return str;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
+/* event callback */
+gboolean
+background_parent_call (const gchar * event_group_name, const gchar * event_name,
+                        gpointer init_data, gpointer data)
+{
+    ev_background_parent_call_t *event_data = (ev_background_parent_call_t *) data;
+
+    (void) event_group_name;
+    (void) event_name;
+    (void) init_data;
+
+    event_data->ret.i =
+        parent_va_call (event_data->routine, event_data->ctx, event_data->argc, event_data->ap);
+
+    return TRUE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+/* event callback */
+gboolean
+background_parent_call_string (const gchar * event_group_name, const gchar * event_name,
+                               gpointer init_data, gpointer data)
+{
+    ev_background_parent_call_t *event_data = (ev_background_parent_call_t *) data;
+
+    (void) event_group_name;
+    (void) event_name;
+    (void) init_data;
+
+    event_data->ret.s =
+        parent_va_call_string (event_data->routine, event_data->argc, event_data->ap);
+
+    return TRUE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 #endif /* WITH_BACKGROUND */
