@@ -12,7 +12,6 @@
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
-#define MAX_MACRO_LENGTH 1024
 #define N_LINE_CACHES 32
 
 /*** enums ***************************************************************************************/
@@ -102,12 +101,19 @@ struct WEdit
     GArray *serialized_bookmarks;
 
     /* undo stack and pointers */
-    unsigned long stack_pointer;
+    unsigned long undo_stack_pointer;
     long *undo_stack;
-    unsigned long stack_size;
-    unsigned long stack_size_mask;
-    unsigned long stack_bottom;
-    unsigned int stack_disable:1;       /* If not 0, don't save events in the undo stack */
+    unsigned long undo_stack_size;
+    unsigned long undo_stack_size_mask;
+    unsigned long undo_stack_bottom;
+    unsigned int undo_stack_disable:1;       /* If not 0, don't save events in the undo stack */
+
+    unsigned long redo_stack_pointer;
+    long *redo_stack;
+    unsigned long redo_stack_size;
+    unsigned long redo_stack_size_mask;
+    unsigned long redo_stack_bottom;
+    unsigned int redo_stack_reset:1;         /* If 1, need clear redo stack */
 
     struct stat stat1;          /* Result of mc_fstat() on the file */
     unsigned int skip_detach_prompt:1;  /* Do not prompt whether to detach a file anymore */
@@ -121,11 +127,6 @@ struct WEdit
     GTree *defines;             /* List of defines */
     gboolean is_case_insensitive;       /* selects language case sensitivity */
 
-    /* macro stuff */
-    int macro_i;                /* index to macro[], -1 if not recording a macro */
-    int macro_depth;            /* depth of the macro recursion */
-    struct macro macro[MAX_MACRO_LENGTH];
-
     /* user map stuff */
     GIConv converter;
 
