@@ -1,24 +1,5 @@
-/* edit.h - editor public API
-
-   Copyright (C) 1996, 1997, 2009 Free Software Foundation, Inc.
-
-   Authors: 1996, 1997 Paul Sheer
-   2009 Andrew Borodin
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+/*
+   Editor public API
  */
 
 /** \file edit.h
@@ -62,6 +43,7 @@ extern int option_line_state;
 extern int option_save_mode;
 extern int option_save_position;
 extern int option_syntax_highlighting;
+extern int option_group_undo;
 extern char *option_backup_ext;
 
 extern int edit_confirm_save;
