@@ -1,24 +1,26 @@
-/* editor menu definitions and initialisation
+/*
+   Editor menu definitions and initialisation
 
-   Copyright (C) 1996, 1998, 2001, 2002, 2003, 2005, 2007
-   Free Software Foundation, Inc.
+   Copyright (C) 1996, 1998, 2001, 2002, 2003, 2005, 2007, 2011
+   The Free Software Foundation, Inc.
 
-   Authors: 1996, 1997 Paul Sheer
+   Written by:
+   Paul Sheer, 1996, 1997
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /** \file
@@ -67,16 +69,16 @@ create_file_menu (void)
 {
     GList *entries = NULL;
 
-    entries = g_list_prepend (entries, menu_entry_create (_("&Open file..."), CK_Load));
-    entries = g_list_prepend (entries, menu_entry_create (_("&New"), CK_New));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Open file..."), CK_EditFile));
+    entries = g_list_prepend (entries, menu_entry_create (_("&New"), CK_EditNew));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries = g_list_prepend (entries, menu_entry_create (_("&Save"), CK_Save));
-    entries = g_list_prepend (entries, menu_entry_create (_("Save &as..."), CK_Save_As));
+    entries = g_list_prepend (entries, menu_entry_create (_("Save &as..."), CK_SaveAs));
     entries = g_list_prepend (entries, menu_separator_create ());
-    entries = g_list_prepend (entries, menu_entry_create (_("&Insert file..."), CK_Insert_File));
-    entries = g_list_prepend (entries, menu_entry_create (_("Cop&y to file..."), CK_Save_Block));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Insert file..."), CK_InsertFile));
+    entries = g_list_prepend (entries, menu_entry_create (_("Cop&y to file..."), CK_BlockSave));
     entries = g_list_prepend (entries, menu_separator_create ());
-    entries = g_list_prepend (entries, menu_entry_create (_("&User menu..."), CK_User_Menu));
+    entries = g_list_prepend (entries, menu_entry_create (_("&User menu..."), CK_UserMenu));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries = g_list_prepend (entries, menu_entry_create (_("A&bout..."), CK_About));
     entries = g_list_prepend (entries, menu_separator_create ());
@@ -93,25 +95,26 @@ create_edit_menu (void)
     GList *entries = NULL;
 
     entries = g_list_prepend (entries, menu_entry_create (_("&Undo"), CK_Undo));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Redo"), CK_Redo));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries =
-        g_list_prepend (entries, menu_entry_create (_("&Toggle ins/overw"), CK_Toggle_Insert));
+        g_list_prepend (entries, menu_entry_create (_("&Toggle ins/overw"), CK_InsertOverwrite));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries = g_list_prepend (entries, menu_entry_create (_("To&ggle mark"), CK_Mark));
-    entries = g_list_prepend (entries, menu_entry_create (_("&Mark columns"), CK_Column_Mark));
-    entries = g_list_prepend (entries, menu_entry_create (_("Mark &all"), CK_Mark_All));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Mark columns"), CK_MarkColumn));
+    entries = g_list_prepend (entries, menu_entry_create (_("Mark &all"), CK_MarkAll));
     entries = g_list_prepend (entries, menu_entry_create (_("Unmar&k"), CK_Unmark));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries = g_list_prepend (entries, menu_entry_create (_("Cop&y"), CK_Copy));
     entries = g_list_prepend (entries, menu_entry_create (_("Mo&ve"), CK_Move));
     entries = g_list_prepend (entries, menu_entry_create (_("&Delete"), CK_Remove));
     entries = g_list_prepend (entries, menu_separator_create ());
-    entries = g_list_prepend (entries, menu_entry_create (_("Co&py to clipfile"), CK_XStore));
-    entries = g_list_prepend (entries, menu_entry_create (_("&Cut to clipfile"), CK_XCut));
-    entries = g_list_prepend (entries, menu_entry_create (_("Pa&ste from clipfile"), CK_XPaste));
+    entries = g_list_prepend (entries, menu_entry_create (_("Co&py to clipfile"), CK_Store));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Cut to clipfile"), CK_Cut));
+    entries = g_list_prepend (entries, menu_entry_create (_("Pa&ste from clipfile"), CK_Paste));
     entries = g_list_prepend (entries, menu_separator_create ());
-    entries = g_list_prepend (entries, menu_entry_create (_("&Beginning"), CK_Beginning_Of_Text));
-    entries = g_list_prepend (entries, menu_entry_create (_("&End"), CK_End_Of_Text));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Beginning"), CK_Top));
+    entries = g_list_prepend (entries, menu_entry_create (_("&End"), CK_Bottom));
 
     return g_list_reverse (entries);
 }
@@ -123,16 +126,14 @@ create_search_replace_menu (void)
 {
     GList *entries = NULL;
 
-    entries = g_list_prepend (entries, menu_entry_create (_("&Search..."), CK_Find));
-    entries = g_list_prepend (entries, menu_entry_create (_("Search &again"), CK_Find_Again));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Search..."), CK_Search));
+    entries = g_list_prepend (entries, menu_entry_create (_("Search &again"), CK_SearchContinue));
     entries = g_list_prepend (entries, menu_entry_create (_("&Replace..."), CK_Replace));
     entries = g_list_prepend (entries, menu_separator_create ());
-    entries =
-        g_list_prepend (entries, menu_entry_create (_("&Toggle bookmark"), CK_Toggle_Bookmark));
-    entries = g_list_prepend (entries, menu_entry_create (_("&Next bookmark"), CK_Next_Bookmark));
-    entries = g_list_prepend (entries, menu_entry_create (_("&Prev bookmark"), CK_Prev_Bookmark));
-    entries =
-        g_list_prepend (entries, menu_entry_create (_("&Flush bookmark"), CK_Flush_Bookmarks));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Toggle bookmark"), CK_Bookmark));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Next bookmark"), CK_BookmarkNext));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Prev bookmark"), CK_BookmarkPrev));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Flush bookmarks"), CK_BookmarkFlush));
 
     return g_list_reverse (entries);
 }
@@ -145,23 +146,18 @@ create_command_menu (void)
     GList *entries = NULL;
 
     entries = g_list_prepend (entries, menu_entry_create (_("&Go to line..."), CK_Goto));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Toggle line state"), CK_ShowNumbers));
     entries =
-        g_list_prepend (entries, menu_entry_create (_("&Toggle line state"), CK_Toggle_Line_State));
+        g_list_prepend (entries, menu_entry_create (_("Go to matching &bracket"), CK_MatchBracket));
     entries =
         g_list_prepend (entries,
-                        menu_entry_create (_("Go to matching &bracket"), CK_Match_Bracket));
-    entries =
-        g_list_prepend (entries,
-                        menu_entry_create (_("Toggle s&yntax highlighting"), CK_Toggle_Syntax));
+                        menu_entry_create (_("Toggle s&yntax highlighting"), CK_SyntaxOnOff));
     entries = g_list_prepend (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("&Find declaration"), CK_Find));
     entries =
-        g_list_prepend (entries, menu_entry_create (_("&Find declaration"), CK_Find_Definition));
+        g_list_prepend (entries, menu_entry_create (_("Back from &declaration"), CK_FilePrev));
     entries =
-        g_list_prepend (entries,
-                        menu_entry_create (_("Back from &declaration"), CK_Load_Prev_File));
-    entries =
-        g_list_prepend (entries,
-                        menu_entry_create (_("For&ward to declaration"), CK_Load_Next_File));
+        g_list_prepend (entries, menu_entry_create (_("For&ward to declaration"), CK_FileNext));
 #ifdef HAVE_CHARSET
     entries = g_list_prepend (entries, menu_separator_create ());
     entries = g_list_prepend (entries, menu_entry_create (_("Encod&ing..."), CK_SelectCodepage));
@@ -171,16 +167,14 @@ create_command_menu (void)
     entries = g_list_prepend (entries, menu_separator_create ());
     entries =
         g_list_prepend (entries,
-                        menu_entry_create (_("&Start record macro"), CK_Begin_Record_Macro));
+                        menu_entry_create (_("&Start/Stop record macro"), CK_MacroStartStopRecord));
+    entries = g_list_prepend (entries, menu_entry_create (_("Delete macr&o..."), CK_MacroDelete));
     entries =
         g_list_prepend (entries,
-                        menu_entry_create (_("Finis&h record macro..."), CK_End_Record_Macro));
-    entries =
-        g_list_prepend (entries, menu_entry_create (_("&Execute macro..."), CK_Execute_Macro));
-    entries = g_list_prepend (entries, menu_entry_create (_("Delete macr&o..."), CK_Delete_Macro));
+                        menu_entry_create (_("Record/Repeat &actions"), CK_RepeatStartStopRecord));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries =
-        g_list_prepend (entries, menu_entry_create (_("'ispell' s&pell check"), CK_Pipe_Block (1)));
+        g_list_prepend (entries, menu_entry_create (_("'ispell' s&pell check"), CK_PipeBlock (1)));
     entries = g_list_prepend (entries, menu_entry_create (_("&Mail..."), CK_Mail));
 
     return g_list_reverse (entries);
@@ -194,15 +188,16 @@ create_format_menu (void)
     GList *entries = NULL;
 
     entries =
-        g_list_prepend (entries, menu_entry_create (_("Insert &literal..."), CK_Insert_Literal));
+        g_list_prepend (entries, menu_entry_create (_("Insert &literal..."), CK_InsertLiteral));
     entries = g_list_prepend (entries, menu_entry_create (_("Insert &date/time"), CK_Date));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries =
-        g_list_prepend (entries, menu_entry_create (_("&Format paragraph"), CK_Paragraph_Format));
+        g_list_prepend (entries, menu_entry_create (_("&Format paragraph"), CK_ParagraphFormat));
     entries = g_list_prepend (entries, menu_entry_create (_("&Sort..."), CK_Sort));
-    entries = g_list_prepend (entries, menu_entry_create (_("&Paste output of..."), CK_ExtCmd));
     entries =
-        g_list_prepend (entries, menu_entry_create (_("&External formatter"), CK_Pipe_Block (0)));
+        g_list_prepend (entries, menu_entry_create (_("&Paste output of..."), CK_ExternalCommand));
+    entries =
+        g_list_prepend (entries, menu_entry_create (_("&External formatter"), CK_PipeBlock (0)));
 
     return g_list_reverse (entries);
 }
@@ -214,17 +209,16 @@ create_options_menu (void)
 {
     GList *entries = NULL;
 
-    entries = g_list_prepend (entries, menu_entry_create (_("&General..."), CK_Edit_Options));
-    entries = g_list_prepend (entries, menu_entry_create (_("Save &mode..."), CK_Edit_Save_Mode));
+    entries = g_list_prepend (entries, menu_entry_create (_("&General..."), CK_Options));
+    entries = g_list_prepend (entries, menu_entry_create (_("Save &mode..."), CK_OptionsSaveMode));
     entries = g_list_prepend (entries, menu_entry_create (_("Learn &keys..."), CK_LearnKeys));
     entries =
-        g_list_prepend (entries,
-                        menu_entry_create (_("Syntax &highlighting..."), CK_Choose_Syntax));
+        g_list_prepend (entries, menu_entry_create (_("Syntax &highlighting..."), CK_SyntaxChoose));
     entries = g_list_prepend (entries, menu_separator_create ());
-    entries = g_list_prepend (entries, menu_entry_create (_("S&yntax file"), CK_Load_Syntax_File));
-    entries = g_list_prepend (entries, menu_entry_create (_("&Menu file"), CK_Load_Menu_File));
+    entries = g_list_prepend (entries, menu_entry_create (_("S&yntax file"), CK_EditSyntaxFile));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Menu file"), CK_EditUserMenu));
     entries = g_list_prepend (entries, menu_separator_create ());
-    entries = g_list_prepend (entries, menu_entry_create (_("&Save setup"), CK_SaveSetupCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Save setup"), CK_SaveSetup));
 
     return g_list_reverse (entries);
 }
