@@ -1,23 +1,28 @@
-/* Panel layout module for the Midnight Commander
+/*
+   Panel layout module for the Midnight Commander
+
    Copyright (C) 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2006, 2007, 2009 Free Software Foundation, Inc.
+   2006, 2007, 2009, 2011
+   The Free Software Foundation, Inc.
 
-   Written: 1995 Janne Kukonlehto
-   1995 Miguel de Icaza
+   Written by:
+   Janne Kukonlehto, 1995
+   Miguel de Icaza, 1995
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /** \file layout.c
@@ -26,22 +31,10 @@
 
 #include <config.h>
 
-#include <signal.h>
-#include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-
-     /*
-      * If TIOCGWINSZ supported, make it available here, because window-
-      * resizing code depends on it...
-      */
-#ifdef HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
-#endif
-#include <termios.h>
 #include <unistd.h>
 
 #include "lib/global.h"
@@ -49,14 +42,12 @@
 #include "lib/skin.h"
 #include "lib/tty/key.h"
 #include "lib/tty/mouse.h"
-#include "lib/tty/win.h"        /* do_enter_ca_mode() */
 #include "lib/mcconfig.h"
-#include "lib/vfs/mc-vfs/vfs.h" /* For vfs_translate_url() */
+#include "lib/vfs/vfs.h"        /* For mc_get_current_wd() */
 #include "lib/strutil.h"
 #include "lib/widget.h"
-#include "lib/util.h"           /* strip_ctrl_codes() */
+#include "lib/event.h"
 
-#include "src/main.h"
 #include "src/consaver/cons.saver.h"
 #include "src/viewer/mcviewer.h"        /* The view widget */
 #include "src/setup.h"
@@ -83,9 +74,6 @@ int nice_rotating_dash = 1;
 /* Set if the panels are split horizontally */
 int horizontal_split = 0;
 
-/* Set if the window has changed it's size */
-int winch_flag = 0;
-
 /* Set if the split is the same */
 int equal_split = 1;
 
@@ -96,19 +84,13 @@ int first_panel_size = 0;
 int output_lines = 0;
 
 /* Set if the command prompt is to be displayed */
-int command_prompt = 1;
+gboolean command_prompt = TRUE;
 
 /* Set if the main menu is visible */
 int menubar_visible = 1;
 
-/* Set if the nice and useful keybar is visible */
-int keybar_visible = 1;
-
-/* Set if the nice message (hint) bar is visible */
-int message_visible = 1;
-
 /* Set to show current working dir in xterm window title */
-int xterm_title = 1;
+gboolean xterm_title = TRUE;
 
 /* Set to show free space on device assigned to current directory */
 int free_space = 1;
@@ -116,6 +98,8 @@ int free_space = 1;
 /* The starting line for the output of the subprogram */
 int output_start_y = 0;
 
+int ok_to_refresh = 1;
+
 /*** file scope macro definitions ****************************************************************/
 
 /* The maximum number of views managed by the set_display_type routine */
@@ -165,10 +149,10 @@ static int _equal_split;
 static int _first_panel_size;
 static int _menubar_visible;
 static int _output_lines;
-static int _command_prompt;
+static gboolean _command_prompt;
 static int _keybar_visible;
 static int _message_visible;
-static int _xterm_title;
+static gboolean _xterm_title;
 static int _free_space;
 
 static int height;
@@ -185,10 +169,9 @@ static struct
     /* *INDENT-OFF* */
     { N_("Show free sp&ace"), &free_space, NULL},
     { N_("&XTerm window title"), &xterm_title, NULL},
-    { N_("H&intbar visible"), &message_visible, NULL},
-    { N_("&Keybar visible"), &keybar_visible, NULL},
+    { N_("H&intbar visible"), &mc_global.message_visible, NULL},
+    { N_("&Keybar visible"), &mc_global.keybar_visible, NULL},
     { N_("Command &prompt"), &command_prompt, NULL},
-    { N_("Show &mini status"), &show_mini_info, NULL},
     { N_("Menu&bar visible"), &menubar_visible, NULL},
     { N_("&Equal split"), &equal_split, NULL}
     /* *INDENT-ON* */
@@ -199,8 +182,6 @@ static int output_lines_label_len;
 
 static WButton *bleft_widget, *bright_widget;
 
-static int ok_to_refresh = 1;
-
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -250,7 +231,7 @@ update_split (const Dlg_head * h)
        it can change due to calling _check_split() as well */
     _check_split ();
 
-    tty_setcolor (check_options[7].widget->state & C_BOOL ? DISABLED_COLOR : COLOR_NORMAL);
+    tty_setcolor (check_options[6].widget->state & C_BOOL ? DISABLED_COLOR : COLOR_NORMAL);
 
     dlg_move (h, 6, 5);
     tty_printf ("%03d", _first_panel_size);
@@ -328,7 +309,7 @@ layout_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *d
         if (old_output_lines != _output_lines)
         {
             old_output_lines = _output_lines;
-            tty_setcolor (console_flag ? COLOR_NORMAL : DISABLED_COLOR);
+            tty_setcolor (mc_global.tty.console_flag != '\0' ? COLOR_NORMAL : DISABLED_COLOR);
             dlg_move (h, 9, 5);
             tty_print_string (output_lines_label);
             dlg_move (h, 9, 5 + 3 + output_lines_label_len);
@@ -337,19 +318,19 @@ layout_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *d
         return MSG_HANDLED;
 
     case DLG_POST_KEY:
-        _menubar_visible = check_options[6].widget->state & C_BOOL;
-        _command_prompt = check_options[5].widget->state & C_BOOL;
+        _menubar_visible = check_options[5].widget->state & C_BOOL;
+        _command_prompt = (check_options[4].widget->state & C_BOOL) != 0;
         _keybar_visible = check_options[3].widget->state & C_BOOL;
         _message_visible = check_options[2].widget->state & C_BOOL;
-        _xterm_title = check_options[1].widget->state & C_BOOL;
+        _xterm_title = (check_options[1].widget->state & C_BOOL) != 0;
         _free_space = check_options[0].widget->state & C_BOOL;
 
-        if (console_flag)
+        if (mc_global.tty.console_flag != '\0')
         {
             int minimum;
             if (_output_lines < 0)
                 _output_lines = 0;
-            height = LINES - _keybar_visible - _command_prompt -
+            height = LINES - _keybar_visible - (_command_prompt ? 1 : 0) -
                 _menubar_visible - _output_lines - _message_visible;
             minimum = MINHEIGHT * (1 + _horizontal_split);
             if (height < minimum)
@@ -359,13 +340,13 @@ layout_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *d
             }
         }
         else
-            height = LINES - _keybar_visible - _command_prompt -
+            height = LINES - _keybar_visible - (_command_prompt ? 1 : 0) -
                 _menubar_visible - _output_lines - _message_visible;
 
         if (old_output_lines != _output_lines)
         {
             old_output_lines = _output_lines;
-            tty_setcolor (console_flag ? COLOR_NORMAL : DISABLED_COLOR);
+            tty_setcolor (mc_global.tty.console_flag != '\0' ? COLOR_NORMAL : DISABLED_COLOR);
             dlg_move (h, 9, 5 + 3 + output_lines_label_len);
             tty_printf ("%02d", _output_lines);
         }
@@ -391,9 +372,9 @@ layout_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *d
             return MSG_HANDLED;
         }
 
-        if (sender == (Widget *) check_options[7].widget)
+        if (sender == (Widget *) check_options[6].widget)
         {
-            _equal_split = check_options[7].widget->state & C_BOOL;
+            _equal_split = check_options[6].widget->state & C_BOOL;
 
             widget_disable (bleft_widget->widget, _equal_split);
             send_message ((Widget *) bleft_widget, WIDGET_DRAW, 0);
@@ -440,8 +421,8 @@ init_layout (void)
     _equal_split = equal_split;
     _menubar_visible = menubar_visible;
     _command_prompt = command_prompt;
-    _keybar_visible = keybar_visible;
-    _message_visible = message_visible;
+    _keybar_visible = mc_global.keybar_visible;
+    _message_visible = mc_global.message_visible;
     _xterm_title = xterm_title;
     _free_space = free_space;
     old_first_panel_size = -1;
@@ -520,7 +501,7 @@ init_layout (void)
 
     /* "Console output" groupbox */
     {
-        const int disabled = console_flag ? 0 : W_DISABLED;
+        const int disabled = mc_global.tty.console_flag != '\0' ? 0 : W_DISABLED;
         Widget *w;
 
         w = (Widget *) button_new (9, output_lines_label_len + 5 + 5, B_MINUS,
@@ -548,8 +529,8 @@ init_layout (void)
     widget_disable (bleft_widget->widget, _equal_split);
     add_widget (layout_dlg, bleft_widget);
 
-    check_options[7].widget = check_new (5, 5, XTRACT (7));
-    add_widget (layout_dlg, check_options[7].widget);
+    check_options[6].widget = check_new (5, 5, XTRACT (6));
+    add_widget (layout_dlg, check_options[6].widget);
 
     radio_widget = radio_new (3, 5, 2, s_split_direction);
     radio_widget->sel = horizontal_split;
@@ -617,49 +598,6 @@ restore_into_right_dir_panel (int idx, Widget * from_widget)
     return new_widget;
 }
 
-/* --------------------------------------------------------------------------------------------- */
-
-static inline void
-low_level_change_screen_size (void)
-{
-#if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#if defined TIOCGWINSZ
-    struct winsize winsz;
-
-    winsz.ws_col = winsz.ws_row = 0;
-    /* Ioctl on the STDIN_FILENO */
-    ioctl (0, TIOCGWINSZ, &winsz);
-    if (winsz.ws_col && winsz.ws_row)
-    {
-#if defined(NCURSES_VERSION) && defined(HAVE_RESIZETERM)
-        resizeterm (winsz.ws_row, winsz.ws_col);
-        clearok (stdscr, TRUE); /* sigwinch's should use a semaphore! */
-#else
-        COLS = winsz.ws_col;
-        LINES = winsz.ws_row;
-#endif
-#ifdef HAVE_SUBSHELL_SUPPORT
-        resize_subshell ();
-#endif
-    }
-#endif /* TIOCGWINSZ */
-#endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-static void
-dlg_resize_cb (void *data, void *user_data)
-{
-    Dlg_head *d = data;
-
-    (void) user_data;
-    if (d->state == DLG_ACTIVE)
-        d->callback (d, NULL, DLG_RESIZE, 0, NULL);
-    else
-        d->winch_pending = TRUE;
-}
-
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -704,57 +642,19 @@ layout_box (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-clr_scr (void)
-{
-    tty_set_normal_attrs ();
-    tty_fill_region (0, 0, LINES, COLS, ' ');
-    tty_refresh ();
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-void
-repaint_screen (void)
-{
-    do_refresh ();
-    tty_refresh ();
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-void
-mc_refresh (void)
-{
-#ifdef WITH_BACKGROUND
-    if (we_are_background)
-        return;
-#endif /* WITH_BACKGROUND */
-    if (winch_flag == 0)
-        tty_refresh ();
-    else
-    {
-        /* if winch was caugth, we should do not only redraw screen, but
-           reposition/resize all */
-        change_screen_size ();
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 setup_panels (void)
 {
     int start_y;
 
-    if (console_flag)
+    if (mc_global.tty.console_flag != '\0')
     {
         int minimum;
         if (output_lines < 0)
             output_lines = 0;
         height =
-            LINES - keybar_visible - command_prompt - menubar_visible -
-            output_lines - message_visible;
+            LINES - mc_global.keybar_visible - (command_prompt ? 1 : 0) - menubar_visible -
+            output_lines - mc_global.message_visible;
         minimum = MINHEIGHT * (1 + horizontal_split);
         if (height < minimum)
         {
@@ -764,7 +664,9 @@ setup_panels (void)
     }
     else
     {
-        height = LINES - menubar_visible - command_prompt - keybar_visible - message_visible;
+        height =
+            LINES - menubar_visible - (command_prompt ? 1 : 0) - mc_global.keybar_visible -
+            mc_global.message_visible;
     }
     check_split ();
     start_y = menubar_visible;
@@ -794,7 +696,7 @@ setup_panels (void)
     if (command_prompt)
     {
 #ifdef HAVE_SUBSHELL_SUPPORT
-        if (!use_subshell || !do_load_prompt ())
+        if (!mc_global.tty.use_subshell || !do_load_prompt ())
 #endif
             setup_cmdline ();
     }
@@ -805,19 +707,19 @@ setup_panels (void)
         widget_set_size (&the_prompt->widget, LINES, COLS, 0, 0);
     }
 
-    widget_set_size (&the_bar->widget, LINES - 1, 0, keybar_visible, COLS);
-    buttonbar_set_visible (the_bar, keybar_visible);
+    widget_set_size (&the_bar->widget, LINES - 1, 0, mc_global.keybar_visible, COLS);
+    buttonbar_set_visible (the_bar, mc_global.keybar_visible);
 
     /* Output window */
-    if (console_flag && output_lines)
+    if (mc_global.tty.console_flag != '\0' && output_lines)
     {
-        output_start_y = LINES - command_prompt - keybar_visible - output_lines;
+        output_start_y = LINES - (command_prompt ? 1 : 0) - mc_global.keybar_visible - output_lines;
         show_console_contents (output_start_y,
-                               LINES - output_lines - keybar_visible - 1,
-                               LINES - keybar_visible - 1);
+                               LINES - output_lines - mc_global.keybar_visible - 1,
+                               LINES - mc_global.keybar_visible - 1);
     }
 
-    if (message_visible)
+    if (mc_global.message_visible)
         widget_set_size (&the_hint->widget, height + start_y, 0, 1, COLS);
     else
         widget_set_size (&the_hint->widget, 0, 0, 0, 0);
@@ -827,56 +729,6 @@ setup_panels (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-sigwinch_handler (int dummy)
-{
-    (void) dummy;
-#if !(defined(USE_NCURSES) || defined(USE_NCURSESW))    /* don't do malloc in a signal handler */
-    low_level_change_screen_size ();
-#endif
-    winch_flag = 1;
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-void
-change_screen_size (void)
-{
-    winch_flag = 0;
-#if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#if defined TIOCGWINSZ
-
-#ifndef NCURSES_VERSION
-    tty_noraw_mode ();
-    tty_reset_screen ();
-#endif
-    low_level_change_screen_size ();
-#ifdef HAVE_SLANG
-    /* XSI Curses spec states that portable applications shall not invoke
-     * initscr() more than once.  This kludge could be done within the scope
-     * of the specification by using endwin followed by a refresh (in fact,
-     * more than one curses implementation does this); it is guaranteed to work
-     * only with slang.
-     */
-    SLsmg_init_smg ();
-    do_enter_ca_mode ();
-    tty_keypad (TRUE);
-    tty_nodelay (FALSE);
-#endif
-
-    /* Inform all suspending dialogs */
-    dialog_switch_got_winch ();
-    /* Inform all running dialogs */
-    g_list_foreach (top_dlg, (GFunc) dlg_resize_cb, NULL);
-
-    /* Now, force the redraw */
-    repaint_screen ();
-#endif /* TIOCGWINSZ */
-#endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 setup_cmdline (void)
 {
@@ -885,7 +737,7 @@ setup_cmdline (void)
     char *tmp_prompt = NULL;
 
 #ifdef HAVE_SUBSHELL_SUPPORT
-    if (use_subshell)
+    if (mc_global.tty.use_subshell)
         tmp_prompt = strip_ctrl_codes (subshell_prompt);
     if (tmp_prompt == NULL)
 #endif
@@ -901,7 +753,7 @@ setup_cmdline (void)
 
     mc_prompt = tmp_prompt;
 
-    y = LINES - 1 - keybar_visible;
+    y = LINES - 1 - mc_global.keybar_visible;
 
     widget_set_size ((Widget *) the_prompt, y, 0, 1, prompt_len);
     label_set_text (the_prompt, mc_prompt);
@@ -932,45 +784,6 @@ set_hintbar (const char *str)
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-print_vfs_message (const char *msg, ...)
-{
-    va_list ap;
-    char str[128];
-
-    va_start (ap, msg);
-    g_vsnprintf (str, sizeof (str), msg, ap);
-    va_end (ap);
-
-    if (midnight_shutdown)
-        return;
-
-    if (!message_visible || !the_hint || !the_hint->widget.owner)
-    {
-        int col, row;
-
-        if (!nice_rotating_dash || (ok_to_refresh <= 0))
-            return;
-
-        /* Preserve current cursor position */
-        tty_getyx (&row, &col);
-
-        tty_gotoyx (0, 0);
-        tty_setcolor (NORMAL_COLOR);
-        tty_print_string (str_fit_to_term (str, COLS - 1, J_LEFT));
-
-        /* Restore cursor position */
-        tty_gotoyx (row, col);
-        mc_refresh ();
-        return;
-    }
-
-    if (message_visible)
-        set_hintbar (str);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 rotate_dash (void)
 {
@@ -1024,9 +837,10 @@ void
 set_display_type (int num, panel_view_mode_t type)
 {
     int x = 0, y = 0, cols = 0, lines = 0;
-    unsigned int the_other = 0;          /* Index to the other panel */
+    unsigned int the_other = 0; /* Index to the other panel */
     const char *file_name = NULL;       /* For Quick view */
     Widget *new_widget = NULL, *old_widget = NULL;
+    panel_view_mode_t old_type = view_listing;
     WPanel *the_other_panel = NULL;
 
     if (num >= MAX_VIEWS)
@@ -1054,15 +868,13 @@ set_display_type (int num, panel_view_mode_t type)
         cols = w->cols;
         lines = w->lines;
         old_widget = w;
+        old_type = panels[num].type;
 
-        if (panels[num].type == view_listing)
+        if (old_type == view_listing && panel->frame_size == frame_full && type != view_listing)
         {
-            if (panel->frame_size == frame_full && type != view_listing)
-            {
-                cols = COLS - first_panel_size;
-                if (num == 1)
-                    x = first_panel_size;
-            }
+            cols = COLS - first_panel_size;
+            if (num == 1)
+                x = first_panel_size;
         }
     }
 
@@ -1114,12 +926,29 @@ set_display_type (int num, panel_view_mode_t type)
     /* We use replace to keep the circular list of the dialog in the */
     /* same state.  Maybe we could just kill it and then replace it  */
     if ((midnight_dlg != NULL) && (old_widget != NULL))
+    {
+        if (old_type == view_listing)
+        {
+            /* save and write directory history of panel
+             * ... and other histories of midnight_dlg  */
+            dlg_save_history (midnight_dlg);
+        }
+
         dlg_replace_widget (old_widget, new_widget);
+    }
 
     if (type == view_listing)
     {
         WPanel *panel = (WPanel *) new_widget;
 
+        /* if existing panel changed type to view_listing, then load history */
+        if (old_widget != NULL)
+        {
+            ev_history_load_save_t event_data = { NULL, new_widget };
+
+            mc_event_raise (midnight_dlg->event_group, MCEVENT_HISTORY_LOAD, &event_data);
+        }
+
         if (num == 0)
             left_panel = panel;
         else
@@ -1145,44 +974,6 @@ set_display_type (int num, panel_view_mode_t type)
 
     g_free (old_widget);
 }
-/* --------------------------------------------------------------------------------------------- */
-
-void
-panel_update_cols (Widget * widget, panel_display_t frame_size)
-{
-    int cols, origin;
-
-    /* don't touch panel if it is not in dialog yet */
-    /* if panel is not in dialog it is not in widgets list
-       and cannot be compared with get_panel_widget() result */
-    if (widget->owner == NULL)
-        return;
-
-    if (horizontal_split)
-    {
-        widget->cols = COLS;
-        return;
-    }
-
-    if (frame_size == frame_full)
-    {
-        cols = COLS;
-        origin = 0;
-    }
-    else if (widget == get_panel_widget (0))
-    {
-        cols = first_panel_size;
-        origin = 0;
-    }
-    else
-    {
-        cols = COLS - first_panel_size;
-        origin = first_panel_size;
-    }
-
-    widget->cols = cols;
-    widget->x = origin;
-}
 
 /* --------------------------------------------------------------------------------------------- */
 /** This routine is deeply sticked to the two panels idea.
@@ -1384,7 +1175,7 @@ save_panel_dir (int idx)
 
         g_free (panels[idx].last_saved_dir);    /* last path no needed */
         /* Because path can be nonlocal */
-        panels[idx].last_saved_dir = vfs_translate_url (widget_work_dir);
+        panels[idx].last_saved_dir = g_strdup (widget_work_dir);
     }
 }
 
