@@ -11,6 +11,8 @@
 
 /*** structures declarations (and typedefs of structures)*****************************************/
 
+struct WPanel;
+
 /*** global variables defined in .c file *********************************************************/
 
 /*** declarations of public functions ************************************************************/
@@ -19,6 +21,8 @@ void external_panelize (void);
 void load_panelize (void);
 void save_panelize (void);
 void done_panelize (void);
+void cd_panelize_cmd (void);
+void panelize_save_panel (struct WPanel *panel);
 
 /*** inline functions ****************************************************************************/
 #endif /* MC__PANELIZE_H */
