@@ -1,34 +1,36 @@
 /*
- * Tree Store
- *
- * Contains a storage of the file system tree representation
- *
- Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2009
- Free Software Foundation, Inc.
+   Tree Store
+   Contains a storage of the file system tree representation
 
- Written: 1994, 1996 Janne Kukonlehto
- 1997 Norbert Warmuth
- 1996, 1999 Miguel de Icaza
+   This module has been converted to be a widget.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
+   The program load and saves the tree each time the tree widget is
+   created and destroyed.  This is required for the future vfs layer,
+   it will be possible to have tree views over virtual file systems.
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2009,
+   2011
+   The Free Software Foundation, Inc.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   Written by:
+   Janne Kukonlehto, 1994, 1996
+   Norbert Warmuth, 1997
+   Miguel de Icaza, 1996, 1999
 
- This module has been converted to be a widget.
+   This file is part of the Midnight Commander.
 
- The program load and saves the tree each time the tree widget is
- created and destroyed.  This is required for the future vfs layer,
- it will be possible to have tree views over virtual file systems.
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /** \file treestore.c
@@ -49,7 +51,7 @@
 
 #include "lib/global.h"
 #include "lib/mcconfig.h"
-#include "lib/vfs/mc-vfs/vfs.h"
+#include "lib/vfs/vfs.h"
 #include "lib/fileloc.h"
 #include "lib/hook.h"
 #include "lib/util.h"
@@ -248,22 +250,26 @@ tree_store_load_from (char *name)
                     different = strtok (NULL, "");
                     if (different)
                     {
+                        vfs_path_t *vpath = vfs_path_from_str (oldname);
                         strcpy (oldname + common, different);
-                        if (vfs_file_is_local (oldname))
+                        if (vfs_file_is_local (vpath))
                         {
                             e = tree_store_add_entry (oldname);
                             e->scanned = scanned;
                         }
+                        vfs_path_free (vpath);
                     }
                 }
             }
             else
             {
-                if (vfs_file_is_local (lc_name))
+                vfs_path_t *vpath = vfs_path_from_str (lc_name);
+                if (vfs_file_is_local (vpath))
                 {
                     e = tree_store_add_entry (lc_name);
                     e->scanned = scanned;
                 }
+                vfs_path_free (vpath);
                 strcpy (oldname, lc_name);
             }
             g_free (lc_name);
@@ -343,8 +349,9 @@ tree_store_save_to (char *name)
     while (current)
     {
         int i, common;
+        vfs_path_t *vpath = vfs_path_from_str (current->name);
 
-        if (vfs_file_is_local (current->name))
+        if (vfs_file_is_local (vpath))
         {
             /* Clear-text compression */
             if (current->prev && (common = str_common (current->prev->name, current->name)) > 2)
@@ -366,9 +373,11 @@ tree_store_save_to (char *name)
             {
                 fprintf (stderr, _("Cannot write to the %s file:\n%s\n"),
                          name, unix_error_string (errno));
+                vfs_path_free (vpath);
                 break;
             }
         }
+        vfs_path_free (vpath);
         current = current->next;
     }
     tree_store_dirty (FALSE);
@@ -638,7 +647,7 @@ tree_store_load (void)
     char *name;
     int retval;
 
-    name = g_build_filename (home_dir, MC_USERCONF_DIR, MC_TREESTORE_FILE, NULL);
+    name = mc_config_get_full_path (MC_TREESTORE_FILE);
     retval = tree_store_load_from (name);
     g_free (name);
 
@@ -658,7 +667,7 @@ tree_store_save (void)
     char *name;
     int retval;
 
-    name = g_build_filename (home_dir, MC_USERCONF_DIR, MC_TREESTORE_FILE, NULL);
+    name = mc_config_get_full_path (MC_TREESTORE_FILE);
     mc_util_make_backup_if_possible (name, ".tmp");
 
     retval = tree_store_save_to (name);
