@@ -1,22 +1,27 @@
-/* Learn keys
+/*
+   Learn keys
+
    Copyright (C) 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2007, 2009 Free Software Foundation, Inc.
+   2007, 2009, 2011
+   The Free Software Foundation, Inc.
 
-   Written by: 1995 Jakub Jelinek
+   Written by:
+   Jakub Jelinek, 1995
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /** \file learn.c
@@ -44,7 +49,7 @@
 #include "lib/util.h"           /* convert_controls() */
 #include "lib/widget.h"
 
-#include "filemanager/layout.h" /* repaint_screen() */
+#include "src/filemanager/layout.h"     /* mc_refresh()  */
 
 #include "setup.h"
 #include "learn.h"
@@ -394,21 +399,24 @@ void
 learn_keys (void)
 {
     int save_old_esc_mode = old_esc_mode;
-    int save_alternate_plus_minus = alternate_plus_minus;
+    gboolean save_alternate_plus_minus = mc_global.tty.alternate_plus_minus;
 
-    old_esc_mode = 0;           /* old_esc_mode cannot work in learn keys dialog */
-    alternate_plus_minus = 1;   /* don't translate KP_ADD, KP_SUBTRACT and
-                                   KP_MULTIPLY to '+', '-' and '*' in
-                                   correct_key_code */
+    /* old_esc_mode cannot work in learn keys dialog */
+    old_esc_mode = 0;
+
+    /* don't translate KP_ADD, KP_SUBTRACT and
+       KP_MULTIPLY to '+', '-' and '*' in
+       correct_key_code */
+    mc_global.tty.alternate_plus_minus = TRUE;
     application_keypad_mode ();
     init_learn ();
 
     run_dlg (learn_dlg);
 
     old_esc_mode = save_old_esc_mode;
-    alternate_plus_minus = save_alternate_plus_minus;
+    mc_global.tty.alternate_plus_minus = save_alternate_plus_minus;
 
-    if (!alternate_plus_minus)
+    if (!mc_global.tty.alternate_plus_minus)
         numeric_keypad_mode ();
 
     switch (learn_dlg->ret_value)
