@@ -1,24 +1,30 @@
-/* Main program for the Midnight Commander
+/*
+   Main program for the Midnight Commander
+
    Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-   2003, 2004, 2005, 2006, 2007, 2009 Free Software Foundation, Inc.
+   2003, 2004, 2005, 2006, 2007, 2009, 2011
+   The Free Software Foundation, Inc.
 
-   Written by: 1994, 1995, 1996, 1997 Miguel de Icaza
-   1994, 1995 Janne Kukonlehto
-   1997 Norbert Warmuth
+   Written by:
+   Miguel de Icaza, 1994, 1995, 1996, 1997
+   Janne Kukonlehto, 1994, 1995
+   Norbert Warmuth, 1997
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** \file main.c
  *  \brief Source: this is a main module
@@ -32,31 +38,32 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/wait.h>
-#include <unistd.h>
 #include <pwd.h>                /* for username in xterm title */
+#include <signal.h>
 
 #include "lib/global.h"
 
+#include "lib/event.h"
 #include "lib/tty/tty.h"
 #include "lib/tty/key.h"        /* For init_key() */
-#include "lib/tty/win.h"        /* xterm_flag */
 #include "lib/skin.h"
 #include "lib/filehighlight.h"
 #include "lib/fileloc.h"
 #include "lib/strutil.h"
 #include "lib/util.h"
-#include "lib/vfs/mc-vfs/vfs.h" /* vfs_init(), vfs_shut() */
+#include "lib/vfs/vfs.h"        /* vfs_init(), vfs_shut() */
 
 #include "filemanager/midnight.h"       /* current_panel */
 #include "filemanager/treestore.h"      /* tree_store_save */
 #include "filemanager/layout.h" /* command_prompt */
 #include "filemanager/ext.h"    /* flush_extension_file() */
 #include "filemanager/command.h"        /* cmdline */
+#include "filemanager/panel.h"          /* panalized_panel */
 
+#include "vfs/plugins_init.h"
+
+#include "events_init.h"
 #include "args.h"
 #include "subshell.h"
 #include "setup.h"              /* load_setup() */
@@ -66,7 +73,7 @@
 #include "selcodepage.h"
 #endif /* HAVE_CHARSET */
 
-#include "consaver/cons.saver.h"        /* console_flag */
+#include "consaver/cons.saver.h"        /* cons_saver_pid */
 
 #include "main.h"
 
@@ -79,59 +86,39 @@ int quit = 0;
 
 #ifdef HAVE_CHARSET
 /* Numbers of (file I/O) and (input/display) codepages. -1 if not selected */
-int source_codepage = -1;
 int default_source_codepage = -1;
-int display_codepage = -1;
 char *autodetect_codeset = NULL;
 gboolean is_autodetect_codeset_enabled = FALSE;
-#else
-/* If true, allow characters in the range 160-255 */
-int eight_bit_clean = 1;
-/*
- * If true, also allow characters in the range 128-159.
- * This is reported to break on many terminals (xterm, qansi-m).
- */
-int full_eight_bits = 0;
 #endif /* !HAVE_CHARSET */
 
-/*
- * If utf-8 terminal utf8_display = 1
- * Display bits set UTF-8
- */
-int utf8_display = 0;
-
 /* If true use the internal viewer */
 int use_internal_view = 1;
 /* If set, use the builtin editor */
 int use_internal_edit = 1;
 
-mc_run_mode_t mc_run_mode = MC_RUN_FULL;
 char *mc_run_param0 = NULL;
 char *mc_run_param1 = NULL;
 
-/* Used so that widgets know if they are being destroyed or
-   shut down */
-int midnight_shutdown = 0;
-
 /* The user's shell */
 char *shell = NULL;
 
 /* The prompt */
 const char *mc_prompt = NULL;
 
-/* mc_home: The home of MC - /etc/mc or defined by MC_DATADIR */
-char *mc_home = NULL;
-/* mc_home_alt: Alternative home of MC - deprecated /usr/share/mc */
-char *mc_home_alt = NULL;
-/* The home directory */
-const char *home_dir = NULL;
-
 /* Set to TRUE to suppress printing the last directory */
 int print_last_revert = FALSE;
 
 /* If set, then print to the given file the last directory we were at */
 char *last_wd_string = NULL;
 
+/* index to record_macro_buf[], -1 if not recording a macro */
+int macro_index = -1;
+
+/* macro stuff */
+struct macro_action_t record_macro_buf[MAX_MACRO_LENGTH];
+
+GArray *macros_list;
+
 /*** file scope macro definitions ****************************************************************/
 
 /*** file scope type declarations ****************************************************************/
@@ -152,20 +139,20 @@ check_codeset (void)
     {
         const char *_display_codepage;
 
-        _display_codepage = get_codepage_id (display_codepage);
+        _display_codepage = get_codepage_id (mc_global.display_codepage);
 
         if (strcmp (_display_codepage, current_system_codepage) != 0)
         {
-            display_codepage = get_codepage_index (current_system_codepage);
-            if (display_codepage == -1)
-                display_codepage = 0;
+            mc_global.display_codepage = get_codepage_index (current_system_codepage);
+            if (mc_global.display_codepage == -1)
+                mc_global.display_codepage = 0;
 
             mc_config_set_string (mc_main_config, "Misc", "display_codepage", cp_display);
         }
     }
 #endif
 
-    utf8_display = str_isutf8 (current_system_codepage);
+    mc_global.utf8_display = str_isutf8 (current_system_codepage);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -175,7 +162,6 @@ static void
 OS_Setup (void)
 {
     const char *shell_env = getenv ("SHELL");
-    const char *mc_libdir;
 
     if ((shell_env == NULL) || (shell_env[0] == '\0'))
     {
@@ -192,26 +178,6 @@ OS_Setup (void)
         g_free (shell);
         shell = g_strdup ("/bin/sh");
     }
-
-    /* This is the directory, where MC was installed, on Unix this is DATADIR */
-    /* and can be overriden by the MC_DATADIR environment variable */
-    mc_libdir = getenv ("MC_DATADIR");
-    if (mc_libdir != NULL)
-    {
-        mc_home = g_strdup (mc_libdir);
-        mc_home_alt = g_strdup (SYSCONFDIR);
-    }
-    else
-    {
-        mc_home = g_strdup (SYSCONFDIR);
-        mc_home_alt = g_strdup (DATADIR);
-    }
-
-    /* This variable is used by the subshell */
-    home_dir = getenv ("HOME");
-
-    if (home_dir == NULL)
-        home_dir = mc_home;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -222,12 +188,12 @@ sigchld_handler_no_subshell (int sig)
 #ifdef __linux__
     int pid, status;
 
-    if (!console_flag)
+    if (!mc_global.tty.console_flag != '\0')
         return;
 
     /* COMMENT: if it were true that after the call to handle_console(..INIT)
-       the value of console_flag never changed, we could simply not install
-       this handler at all if (!console_flag && !use_subshell). */
+       the value of mc_global.tty.console_flag never changed, we could simply not install
+       this handler at all if (!mc_global.tty.console_flag && !mc_global.tty.use_subshell). */
 
     /* That comment is no longer true.  We need to wait() on a sigchld
        handler (that's at least what the tarfs code expects currently). */
@@ -246,7 +212,7 @@ sigchld_handler_no_subshell (int sig)
         {
             /* cons.saver has died - disable console saving */
             handle_console (CONSOLE_DONE);
-            console_flag = 0;
+            mc_global.tty.console_flag = '\0';
         }
     }
     /* If we got here, some other child exited; ignore it */
@@ -264,7 +230,7 @@ init_sigchld (void)
 
     sigchld_action.sa_handler =
 #ifdef HAVE_SUBSHELL_SUPPORT
-        use_subshell ? sigchld_handler :
+        mc_global.tty.use_subshell ? sigchld_handler :
 #endif /* HAVE_SUBSHELL_SUPPORT */
         sigchld_handler_no_subshell;
 
@@ -283,7 +249,7 @@ init_sigchld (void)
          * This may happen on QNX Neutrino 6, where SA_RESTART
          * is defined but not implemented.  Fallback to no subshell.
          */
-        use_subshell = 0;
+        mc_global.tty.use_subshell = FALSE;
 #endif /* HAVE_SUBSHELL_SUPPORT */
     }
 }
@@ -292,32 +258,29 @@ init_sigchld (void)
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-/** Define this function for glib-style error handling */
-GQuark
-mc_main_error_quark (void)
-{
-    return g_quark_from_static_string (PACKAGE);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 int
 do_cd (const char *new_dir, enum cd_enum exact)
 {
     gboolean res;
+    const char *_new_dir = new_dir;
 
-    res = do_panel_cd (current_panel, new_dir, exact);
+    if (current_panel->is_panelized && _new_dir[0] == '.' && _new_dir[1] == '.' && _new_dir[2] == 0)
+        _new_dir = panelized_panel.root;
+
+    res = do_panel_cd (current_panel, _new_dir, exact);
 
 #if HAVE_CHARSET
     if (res)
     {
-        const char *enc_name;
+        vfs_path_t *vpath = vfs_path_from_str (current_panel->cwd);
+        vfs_path_element_t *path_element = vfs_path_get_by_index (vpath, -1);
 
-        enc_name = vfs_get_encoding (current_panel->cwd);
-        if (enc_name != NULL)
-            current_panel->codepage = get_codepage_index (enc_name);
+        if (path_element->encoding != NULL)
+            current_panel->codepage = get_codepage_index (path_element->encoding);
         else
             current_panel->codepage = SELECT_CHARSET_NO_TRANSLATE;
+
+        vfs_path_free (vpath);
     }
 #endif /* HAVE_CHARSET */
 
@@ -380,7 +343,7 @@ update_xterm_title_path (void)
     char *login = NULL;
     int res = 0;
 
-    if (xterm_flag && xterm_title)
+    if (mc_global.tty.xterm_flag && xterm_title)
     {
         path = strip_home_and_password (current_panel->cwd);
         res = gethostname (host, sizeof (host));
@@ -405,9 +368,9 @@ update_xterm_title_path (void)
         fprintf (stdout, "\33]0;%s\7", str_term_form (p));
         g_free (login);
         g_free (p);
-        if (!alternate_plus_minus)
+        if (!mc_global.tty.alternate_plus_minus)
             numeric_keypad_mode ();
-        fflush (stdout);
+        (void) fflush (stdout);
     }
 }
 
@@ -416,34 +379,49 @@ update_xterm_title_path (void)
 int
 main (int argc, char *argv[])
 {
-    struct stat s;
-    char *mc_dir;
     GError *error = NULL;
     gboolean isInitialized;
 
     /* We had LC_CTYPE before, LC_ALL includs LC_TYPE as well */
-    setlocale (LC_ALL, "");
-    bindtextdomain ("mc", LOCALEDIR);
-    textdomain ("mc");
+    (void) setlocale (LC_ALL, "");
+    (void) bindtextdomain ("mc", LOCALEDIR);
+    (void) textdomain ("mc");
+
+    if (!events_init (&error))
+    {
+        fprintf (stderr, _("Failed to run:\n%s\n"), error->message);
+        g_error_free (error);
+        (void) mc_event_deinit (NULL);
+        exit (EXIT_FAILURE);
+    }
 
     /* Set up temporary directory */
-    mc_tmpdir ();
+    (void) mc_tmpdir ();
 
     OS_Setup ();
 
     str_init_strings (NULL);
 
+    /* Initialize and create home directories */
+    /* do it after the screen library initialization to show the error message */
+    mc_config_init_config_paths (&error);
+
+    if (error == NULL && mc_config_deprecated_dir_present ())
+        mc_config_migrate_from_old_place (&error);
+
     vfs_init ();
+    vfs_plugins_init ();
+    vfs_setup_work_dir ();
 
     if (!mc_args_handle (argc, argv, "mc"))
         exit (EXIT_FAILURE);
 
     /* check terminal type
      * $TEMR must be set and not empty
-     * xterm_flag is used in init_key() and tty_init()
+     * mc_global.tty.xterm_flag is used in init_key() and tty_init()
      * Do this after mc_args_handle() where mc_args__force_xterm is set up.
      */
-    xterm_flag = tty_check_term (mc_args__force_xterm);
+    mc_global.tty.xterm_flag = tty_check_term (mc_args__force_xterm);
 
     /* NOTE: This has to be called before tty_init or whatever routine
        calls any define_sequence */
@@ -454,10 +432,10 @@ main (int argc, char *argv[])
 
 #ifdef HAVE_SUBSHELL_SUPPORT
     /* Don't use subshell when invoked as viewer or editor */
-    if (mc_run_mode != MC_RUN_FULL)
-        use_subshell = 0;
+    if (mc_global.mc_run_mode != MC_RUN_FULL)
+        mc_global.tty.use_subshell = FALSE;
 
-    if (use_subshell)
+    if (mc_global.tty.use_subshell)
         subshell_get_console_attributes ();
 #endif /* HAVE_SUBSHELL_SUPPORT */
 
@@ -469,56 +447,61 @@ main (int argc, char *argv[])
 
     /* Must be done before init_subshell, to set up the terminal size: */
     /* FIXME: Should be removed and LINES and COLS computed on subshell */
-    tty_init (mc_args__slow_terminal, mc_args__ugly_line_drawing, !mc_args__nomouse,
-              (gboolean) xterm_flag);
+    tty_init (!mc_args__nomouse, mc_global.tty.xterm_flag);
 
     load_setup ();
 
-    /* start check display_codepage and source_codepage */
+    /* start check mc_global.display_codepage and mc_global.source_codepage */
     check_codeset ();
 
     /* Removing this from the X code let's us type C-c */
     load_key_defs ();
 
-    load_keymap_defs ();
+    load_keymap_defs (!mc_args__nokeymap);
 
-    tty_init_colors (mc_args__disable_colors, mc_args__force_colors);
-    isInitialized = mc_skin_init (&error);
-    mc_filehighlight = mc_fhl_new (TRUE);
-    dlg_set_default_colors ();
+    macros_list = g_array_new (TRUE, FALSE, sizeof (macros_t));
 
-    if (!isInitialized)
+    tty_init_colors (mc_global.tty.disable_colors, mc_args__force_colors);
+
+    {
+        GError *error2 = NULL;
+        isInitialized = mc_skin_init (&error2);
+        mc_filehighlight = mc_fhl_new (TRUE);
+        dlg_set_default_colors ();
+
+        if (!isInitialized)
+        {
+            message (D_ERROR, _("Warning"), "%s", error2->message);
+            g_error_free (error2);
+            error2 = NULL;
+        }
+    }
+
+    if (error != NULL)
     {
         message (D_ERROR, _("Warning"), "%s", error->message);
         g_error_free (error);
         error = NULL;
     }
 
-    /* create home directory */
-    /* do it after the screen library initialization to show the error message */
-    mc_dir = g_build_filename (home_dir, MC_USERCONF_DIR, (char *) NULL);
-    canonicalize_pathname (mc_dir);
-    if ((stat (mc_dir, &s) != 0) && (errno == ENOENT) && mkdir (mc_dir, 0700) != 0)
-        message (D_ERROR, _("Warning"), _("Cannot create %s directory"), mc_dir);
-    g_free (mc_dir);
 
 #ifdef HAVE_SUBSHELL_SUPPORT
     /* Done here to ensure that the subshell doesn't  */
     /* inherit the file descriptors opened below, etc */
-    if (use_subshell)
+    if (mc_global.tty.use_subshell)
         init_subshell ();
 
 #endif /* HAVE_SUBSHELL_SUPPORT */
 
     /* Also done after init_subshell, to save any shell init file messages */
-    if (console_flag)
+    if (mc_global.tty.console_flag != '\0')
         handle_console (CONSOLE_SAVE);
 
-    if (alternate_plus_minus)
+    if (mc_global.tty.alternate_plus_minus)
         application_keypad_mode ();
 
 #ifdef HAVE_SUBSHELL_SUPPORT
-    if (use_subshell)
+    if (mc_global.tty.use_subshell)
     {
         mc_prompt = strip_ctrl_codes (subshell_prompt);
         if (mc_prompt == NULL)
@@ -529,11 +512,11 @@ main (int argc, char *argv[])
         mc_prompt = (geteuid () == 0) ? "# " : "$ ";
 
     /* Program main loop */
-    if (!midnight_shutdown)
+    if (!mc_global.widget.midnight_shutdown)
         do_nc ();
 
     /* Save the tree store */
-    tree_store_save ();
+    (void) tree_store_save ();
 
     free_keymap_defs ();
 
@@ -550,17 +533,17 @@ main (int argc, char *argv[])
 
     done_setup ();
 
-    if (console_flag && (quit & SUBSHELL_EXIT) == 0)
+    if (mc_global.tty.console_flag != '\0' && (quit & SUBSHELL_EXIT) == 0)
         handle_console (CONSOLE_RESTORE);
-    if (alternate_plus_minus)
+    if (mc_global.tty.alternate_plus_minus)
         numeric_keypad_mode ();
 
-    signal (SIGCHLD, SIG_DFL);  /* Disable the SIGCHLD handler */
+    (void) signal (SIGCHLD, SIG_DFL);   /* Disable the SIGCHLD handler */
 
-    if (console_flag)
+    if (mc_global.tty.console_flag != '\0')
         handle_console (CONSOLE_DONE);
 
-    if (mc_run_mode == MC_RUN_FULL && mc_args__last_wd_file != NULL
+    if (mc_global.mc_run_mode == MC_RUN_FULL && mc_args__last_wd_file != NULL
         && last_wd_string != NULL && !print_last_revert)
     {
         int last_wd_fd;
@@ -577,18 +560,40 @@ main (int argc, char *argv[])
     }
     g_free (last_wd_string);
 
-    g_free (mc_home_alt);
-    g_free (mc_home);
     g_free (shell);
 
     done_key ();
 
+    if (macros_list != NULL)
+    {
+        guint i;
+        macros_t *macros;
+        for (i = 0; i < macros_list->len; i++)
+        {
+            macros = &g_array_index (macros_list, struct macros_t, i);
+            if (macros != NULL && macros->macro != NULL)
+                (void) g_array_free (macros->macro, FALSE);
+        }
+        (void) g_array_free (macros_list, TRUE);
+    }
+
     str_uninit_strings ();
 
     g_free (mc_run_param0);
     g_free (mc_run_param1);
 
-    putchar ('\n');             /* Hack to make shell's prompt start at left of screen */
+    (void) mc_event_deinit (&error);
+
+    mc_config_deinit_config_paths ();
+
+    if (error != NULL)
+    {
+        fprintf (stderr, _("\nFailed while close:\n%s\n"), error->message);
+        g_error_free (error);
+        exit (EXIT_FAILURE);
+    }
+
+    (void) putchar ('\n');      /* Hack to make shell's prompt start at left of screen */
 
     return 0;
 }
