@@ -1,20 +1,25 @@
-/* Setup loading/saving.
+/*
+   Setup loading/saving.
+
    Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2006, 2007, 2009, 2010 Free Software Foundation, Inc.
+   2006, 2007, 2009, 2010, 2011
+   The Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** \file setup.c
  *  \brief Source: setup loading/saving
@@ -35,18 +40,18 @@
 #include "lib/mcconfig.h"
 #include "lib/fileloc.h"
 #include "lib/timefmt.h"
-
-#include "lib/vfs/mc-vfs/vfs.h"
-
-#ifdef ENABLE_VFS_FTP
-#include "lib/vfs/mc-vfs/ftpfs.h"
-#endif
-#ifdef ENABLE_VFS_FISH
-#include "lib/vfs/mc-vfs/fish.h"
-#endif
-
 #include "lib/util.h"
 #include "lib/widget.h"
+
+#include "lib/vfs/vfs.h"
+
+#ifdef ENABLE_VFS_FTP
+#include "src/vfs/ftpfs/ftpfs.h"
+#endif
+#ifdef ENABLE_VFS_FISH
+#include "src/vfs/fish/fish.h"
+#endif
+
 #ifdef HAVE_CHARSET
 #include "lib/charsets.h"
 #endif
@@ -78,16 +83,12 @@
 
 /*** global variables ****************************************************************************/
 
-char *profile_name;             /* .mc/ini */
+char *profile_name;             /* ${XDG_CONFIG_HOME}/mc/ini */
 char *global_profile_name;      /* mc.lib */
 
 /* Only used at program boot */
 gboolean boot_current_is_left = TRUE;
 
-char *setup_color_string;
-char *term_color_string;
-char *color_terminal_string;
-
 /* If on, default for "No" in delete operations */
 int safe_delete = 0;
 
@@ -104,8 +105,6 @@ int confirm_overwrite = 1;
 int confirm_execute = 0;
 /* Asks for confirmation before leaving the program */
 int confirm_exit = 1;
-/* Asks for confirmation before clean up of history */
-int confirm_history_cleanup = 1;
 
 /* If true, at startup the user-menu is invoked */
 int auto_menu = 0;
@@ -129,6 +128,7 @@ int option_tab_spacing = DEFAULT_TAB_SPACING;
 int saving_setup;
 
 panels_options_t panels_options = {
+    .show_mini_info = TRUE,
     .kilobyte_si = FALSE,
     .mix_all_files = FALSE,
     .show_backups = TRUE,
@@ -143,7 +143,8 @@ panels_options_t panels_options = {
     .mouse_move_pages = TRUE,
     .filetype_mode = TRUE,
     .permission_mode = FALSE,
-    .qsearch_mode = QSEARCH_PANEL_CASE
+    .qsearch_mode = QSEARCH_PANEL_CASE,
+    .torben_fj_mode = FALSE
 };
 
 int easy_patterns = 1;
@@ -156,12 +157,6 @@ int auto_save_setup = 1;
  */
 int only_leading_plus_minus = 1;
 
-/* Set when cd symlink following is desirable (bash mode) */
-int cd_symlinks = 1;
-
-/* Set if you want the possible completions dialog for the first time */
-int show_all_if_ambiguous = 0;
-
 /* Automatically fills name with current selected item name on mkdir */
 int auto_fill_mkdir_name = 1;
 
@@ -190,7 +185,7 @@ int file_op_compute_totals = 1;
 
 /*** file scope variables ************************************************************************/
 
-static char *panels_profile_name = NULL;        /* .mc/panels.ini */
+static char *panels_profile_name = NULL;        /* ${XDG_CACHE_HOME}/mc/panels.ini */
 
 /* *INDENT-OFF* */
 static const struct
@@ -224,13 +219,12 @@ static const struct
 } layout [] = {
     { "equal_split", &equal_split },
     { "first_panel_size", &first_panel_size },
-    { "message_visible", &message_visible },
-    { "keybar_visible", &keybar_visible },
+    { "message_visible", &mc_global.message_visible },
+    { "keybar_visible", &mc_global.keybar_visible },
     { "xterm_title", &xterm_title },
     { "output_lines", &output_lines },
     { "command_prompt", &command_prompt },
     { "menubar_visible", &menubar_visible },
-    { "show_mini_info", &show_mini_info },
     { "free_space", &free_space },
     { NULL, NULL }
 };
@@ -244,6 +238,7 @@ static const struct
     { "pause_after_run", &pause_after_run },
     { "shell_patterns", &easy_patterns },
     { "auto_save_setup", &auto_save_setup },
+    { "preallocate_space", &mc_global.vfs.preallocate_space },
     { "auto_menu", &auto_menu },
     { "use_internal_view", &use_internal_view },
     { "use_internal_edit", &use_internal_edit },
@@ -251,15 +246,15 @@ static const struct
     { "confirm_delete", &confirm_delete },
     { "confirm_overwrite", &confirm_overwrite },
     { "confirm_execute", &confirm_execute },
-    { "confirm_history_cleanup", &confirm_history_cleanup },
+    { "confirm_history_cleanup", &mc_global.widget.confirm_history_cleanup },
     { "confirm_exit", &confirm_exit },
     { "confirm_directory_hotlist_delete", &confirm_directory_hotlist_delete },
     { "safe_delete", &safe_delete },
     { "mouse_repeat_rate", &mou_auto_repeat },
     { "double_click_speed", &double_click_speed },
 #ifndef HAVE_CHARSET
-    { "eight_bit_clean", &eight_bit_clean },
-    { "full_eight_bits", &full_eight_bits },
+    { "eight_bit_clean", &mc_global.eight_bit_clean },
+    { "full_eight_bits", &mc_global.full_eight_bits },
 #endif /* !HAVE_CHARSET */
     { "use_8th_bit_as_meta", &use_8th_bit_as_meta },
     { "confirm_view_dir", &confirm_view_dir },
@@ -270,12 +265,11 @@ static const struct
     { "wrap_mode",  &mcview_global_wrap_mode},
     { "old_esc_mode", &old_esc_mode },
     { "old_esc_mode_timeout", &old_esc_mode_timeout },
-    { "cd_symlinks", &cd_symlinks },
-    { "show_all_if_ambiguous", &show_all_if_ambiguous },
+    { "cd_symlinks", &mc_global.vfs.cd_symlinks },
+    { "show_all_if_ambiguous", &mc_global.widget.show_all_if_ambiguous },
     { "max_dirt_limit", &mcview_max_dirt_limit },
-    { "torben_fj_mode", &torben_fj_mode },
     { "use_file_to_guess_type", &use_file_to_check_type },
-    { "alternate_plus_minus", &alternate_plus_minus },
+    { "alternate_plus_minus", &mc_global.tty.alternate_plus_minus },
     { "only_leading_plus_minus", &only_leading_plus_minus },
     { "show_output_starts_shell", &output_starts_shell },
     { "xtree_mode", &xtree_mode },
@@ -320,6 +314,7 @@ static const struct
     { "editor_simple_statusbar", &simple_statusbar },
     { "editor_check_new_line", &option_check_nl_at_eof },
     { "editor_show_right_margin", &show_right_margin },
+    { "editor_group_undo", &option_group_undo },
 #endif /* USE_INTERNAL_EDIT */
     { "nice_rotating_dash", &nice_rotating_dash },
     { "horizontal_split",   &horizontal_split },
@@ -346,24 +341,25 @@ static const struct
 static const struct
 {
     const char *opt_name;
-    const char *opt_old_name;
     gboolean *opt_addr;
 } panels_ini_options[] = {
-    { "kilobyte_si",  NULL, &panels_options.kilobyte_si },
-    { "mix_all_files", NULL, &panels_options.mix_all_files },
-    { "show_backups",  NULL, &panels_options.show_backups },
-    { "show_dot_files",  NULL, &panels_options.show_dot_files },
-    { "fast_reload",  NULL, &panels_options.fast_reload },
-    { "fast_reload_msg_shown",  NULL, &panels_options.fast_reload_msg_shown },
-    { "mark_moves_down",  NULL, &panels_options.mark_moves_down },
-    { "reverse_files_only", NULL, &panels_options.reverse_files_only },
-    { "auto_save_setup_panels", "auto_save_setup", &panels_options.auto_save_setup },
-    { "navigate_with_arrows",  NULL, &panels_options.navigate_with_arrows },
-    { "panel_scroll_pages",  "scroll_pages", &panels_options.scroll_pages },
-    { "mouse_move_pages",  NULL, &panels_options.mouse_move_pages },
-    { "filetype_mode",  NULL, &panels_options.filetype_mode },
-    { "permission_mode",  NULL, &panels_options.permission_mode },
-    { NULL, NULL, NULL }
+    { "show_mini_info", &panels_options.show_mini_info },
+    { "kilobyte_si", &panels_options.kilobyte_si },
+    { "mix_all_files", &panels_options.mix_all_files },
+    { "show_backups", &panels_options.show_backups },
+    { "show_dot_files", &panels_options.show_dot_files },
+    { "fast_reload", &panels_options.fast_reload },
+    { "fast_reload_msg_shown", &panels_options.fast_reload_msg_shown },
+    { "mark_moves_down", &panels_options.mark_moves_down },
+    { "reverse_files_only", &panels_options.reverse_files_only },
+    { "auto_save_setup_panels", &panels_options.auto_save_setup },
+    { "navigate_with_arrows", &panels_options.navigate_with_arrows },
+    { "panel_scroll_pages", &panels_options.scroll_pages },
+    { "mouse_move_pages",  &panels_options.mouse_move_pages },
+    { "filetype_mode", &panels_options.filetype_mode },
+    { "permission_mode", &panels_options.permission_mode },
+    { "torben_fj_mode", &panels_options.torben_fj_mode },
+    { NULL, NULL }
 };
 /* *INDENT-ON* */
 
@@ -382,15 +378,13 @@ static const struct
  \return
  Newly allocated path to config name or NULL if file not found.
 
- If config_file_name is a relative path, then search config in stantart pathes.
+ If config_file_name is a relative path, then search config in stantart paths.
 */
 static char *
 load_setup_get_full_config_name (const char *subdir, const char *config_file_name)
 {
     /*
        TODO: IMHO, in future this function must be placed into mc_config module.
-       Also, need to rename stupid mc_home and mc_home_alt to mc_sysconfdir and mc_datadir;
-       home_mc => mc_user_homedir
      */
     char *lc_basename, *ret;
 
@@ -406,9 +400,9 @@ load_setup_get_full_config_name (const char *subdir, const char *config_file_nam
         return NULL;
 
     if (subdir != NULL)
-        ret = g_build_filename (home_dir, MC_USERCONF_DIR, subdir, lc_basename, NULL);
+        ret = g_build_filename (mc_config_get_path (), subdir, lc_basename, NULL);
     else
-        ret = g_build_filename (home_dir, MC_USERCONF_DIR, lc_basename, NULL);
+        ret = g_build_filename (mc_config_get_path (), lc_basename, NULL);
 
     if (exist_file (ret))
     {
@@ -418,9 +412,9 @@ load_setup_get_full_config_name (const char *subdir, const char *config_file_nam
     g_free (ret);
 
     if (subdir != NULL)
-        ret = g_build_filename (mc_home, subdir, lc_basename, NULL);
+        ret = g_build_filename (mc_global.sysconfig_dir, subdir, lc_basename, NULL);
     else
-        ret = g_build_filename (mc_home, lc_basename, NULL);
+        ret = g_build_filename (mc_global.sysconfig_dir, lc_basename, NULL);
 
     if (exist_file (ret))
     {
@@ -430,9 +424,9 @@ load_setup_get_full_config_name (const char *subdir, const char *config_file_nam
     g_free (ret);
 
     if (subdir != NULL)
-        ret = g_build_filename (mc_home_alt, subdir, lc_basename, NULL);
+        ret = g_build_filename (mc_global.share_data_dir, subdir, lc_basename, NULL);
     else
-        ret = g_build_filename (mc_home_alt, lc_basename, NULL);
+        ret = g_build_filename (mc_global.share_data_dir, lc_basename, NULL);
 
     g_free (lc_basename);
 
@@ -517,7 +511,6 @@ setup__move_panels_config_into_separate_file (const char *profile)
 /**
   Create new mc_config object from specified ini-file or
   append data to existing mc_config object from ini-file
-
 */
 
 static void
@@ -529,7 +522,7 @@ load_setup_init_config_from_file (mc_config_t ** config, const char *fname)
     if (exist_file (fname))
     {
         if (*config != NULL)
-            mc_config_read_file (*config, fname);
+            mc_config_read_file (*config, fname, TRUE);
         else
             *config = mc_config_init (fname);
     }
@@ -617,10 +610,7 @@ static void
 load_keymap_from_section (const char *section_name, GArray * keymap, mc_config_t * cfg)
 {
     gchar **profile_keys, **keys;
-    gchar **values, **curr_values;
-    char *valcopy, *value;
-    int action;
-    gsize len, values_len;
+    gsize len;
 
     if (section_name == NULL)
         return;
@@ -629,67 +619,63 @@ load_keymap_from_section (const char *section_name, GArray * keymap, mc_config_t
 
     while (*profile_keys != NULL)
     {
-        curr_values = values =
-            mc_config_get_string_list (cfg, section_name, *profile_keys, &values_len);
+        gchar **values, **curr_values;
 
-        action = keybind_lookup_action (*profile_keys);
+        curr_values = values = mc_config_get_string_list (cfg, section_name, *profile_keys, &len);
 
-        if (action > 0)
+        if (curr_values != NULL)
         {
-            if (curr_values != NULL)
-            {
+            int action;
+
+            action = keybind_lookup_action (*profile_keys);
+            if (action > 0)
                 while (*curr_values != NULL)
                 {
-                    valcopy = convert_controls (*curr_values);
-                    keybind_cmd_bind (keymap, valcopy, action);
-                    g_free (valcopy);
+                    keybind_cmd_bind (keymap, *curr_values, action);
                     curr_values++;
                 }
-            }
-            else
-            {
-                value = mc_config_get_string (cfg, section_name, *profile_keys, "");
-                valcopy = convert_controls (value);
-                /* define_sequence (key_code, valcopy, MCKEY_NOACTION); */
-                g_free (valcopy);
-                g_free (value);
-            }
+
+            g_strfreev (values);
         }
 
         profile_keys++;
-        g_strfreev (values);
     }
+
     g_strfreev (keys);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static mc_config_t *
-load_setup_get_keymap_profile_config (void)
+load_setup_get_keymap_profile_config (gboolean load_from_file)
 {
     /*
        TODO: IMHO, in future this function must be placed into mc_config module.
      */
-    mc_config_t *keymap_config = NULL;
+    mc_config_t *keymap_config;
     char *fname, *fname2;
 
-    /* 1) /usr/share/mc (mc_home_alt) */
-    fname = g_build_filename (mc_home_alt, GLOBAL_KEYMAP_FILE, NULL);
+    /* 0) Create default keymap */
+    keymap_config = create_default_keymap ();
+    if (!load_from_file)
+        return keymap_config;
+
+    /* 1) /usr/share/mc (mc_global.share_data_dir) */
+    fname = g_build_filename (mc_global.share_data_dir, GLOBAL_KEYMAP_FILE, NULL);
     load_setup_init_config_from_file (&keymap_config, fname);
     g_free (fname);
 
-    /* 2) /etc/mc (mc_home) */
-    fname = g_build_filename (mc_home, GLOBAL_KEYMAP_FILE, NULL);
+    /* 2) /etc/mc (mc_global.sysconfig_dir) */
+    fname = g_build_filename (mc_global.sysconfig_dir, GLOBAL_KEYMAP_FILE, NULL);
     load_setup_init_config_from_file (&keymap_config, fname);
     g_free (fname);
 
-    /* 3) ~/.mc (home_dir?) */
-    fname = g_build_filename (home_dir, MC_USERCONF_DIR, GLOBAL_KEYMAP_FILE, NULL);
+    /* 3) ${XDG_CONFIG_HOME}/mc */
+    fname = mc_config_get_full_path (GLOBAL_KEYMAP_FILE);
     load_setup_init_config_from_file (&keymap_config, fname);
     g_free (fname);
 
     /* 4) main config; [Midnight Commander] -> keymap */
-
     fname2 =
         mc_config_get_string (mc_main_config, CONFIG_APP_SECTION, "keymap", GLOBAL_KEYMAP_FILE);
     fname = load_setup_get_full_config_name (NULL, fname2);
@@ -766,7 +752,7 @@ save_panel_types (void)
 {
     panel_view_mode_t type;
 
-    if (mc_run_mode != MC_RUN_FULL)
+    if (mc_global.mc_run_mode != MC_RUN_FULL)
         return;
 
     type = get_display_type (0);
@@ -806,10 +792,10 @@ setup_init (void)
     if (profile_name != NULL)
         return profile_name;
 
-    profile = g_build_filename (home_dir, MC_USERCONF_DIR, MC_CONFIG_FILE, NULL);
+    profile = mc_config_get_full_path (MC_CONFIG_FILE);
     if (!exist_file (profile))
     {
-        inifile = concat_dir_and_file (mc_home, "mc.ini");
+        inifile = concat_dir_and_file (mc_global.sysconfig_dir, "mc.ini");
         if (exist_file (inifile))
         {
             g_free (profile);
@@ -818,7 +804,7 @@ setup_init (void)
         else
         {
             g_free (inifile);
-            inifile = concat_dir_and_file (mc_home_alt, "mc.ini");
+            inifile = concat_dir_and_file (mc_global.share_data_dir, "mc.ini");
             if (exist_file (inifile))
             {
                 g_free (profile);
@@ -851,15 +837,17 @@ load_setup (void)
     profile = setup_init ();
 
     /* mc.lib is common for all users, but has priority lower than
-       ~/.mc/ini.  FIXME: it's only used for keys and treestore now */
-    global_profile_name = g_build_filename (mc_home, MC_GLOBAL_CONFIG_FILE, (char *) NULL);
+       ${XDG_CONFIG_HOME}/mc/ini.  FIXME: it's only used for keys and treestore now */
+    global_profile_name =
+        g_build_filename (mc_global.sysconfig_dir, MC_GLOBAL_CONFIG_FILE, (char *) NULL);
     if (!exist_file (global_profile_name))
     {
         g_free (global_profile_name);
-        global_profile_name = g_build_filename (mc_home_alt, MC_GLOBAL_CONFIG_FILE, (char *) NULL);
+        global_profile_name =
+            g_build_filename (mc_global.share_data_dir, MC_GLOBAL_CONFIG_FILE, (char *) NULL);
     }
 
-    panels_profile_name = g_build_filename (home_dir, MC_USERCONF_DIR, MC_PANELS_FILE, NULL);
+    panels_profile_name = mc_config_get_full_path (MC_PANELS_FILE);
 
     mc_main_config = mc_config_init (profile);
 
@@ -906,11 +894,14 @@ load_setup (void)
 
     if (mc_run_param1 == NULL)
     {
+        vfs_path_t *vpath;
         buffer = mc_config_get_string (mc_panels_config, "Dirs", "other_dir", ".");
-        if (vfs_file_is_local (buffer))
+        vpath = vfs_path_from_str (buffer);
+        if (vfs_file_is_local (vpath))
             mc_run_param1 = buffer;
         else
             g_free (buffer);
+        vfs_path_free (vpath);
     }
 
     boot_current_is_left = mc_config_get_bool (mc_panels_config, "Dirs", "current_is_left", TRUE);
@@ -928,9 +919,12 @@ load_setup (void)
 #endif /* ENABLE_VFS_FTP */
 
     /* The default color and the terminal dependent color */
-    setup_color_string = mc_config_get_string (mc_main_config, "Colors", "base_color", "");
-    term_color_string = mc_config_get_string (mc_main_config, "Colors", getenv ("TERM"), "");
-    color_terminal_string = mc_config_get_string (mc_main_config, "Colors", "color_terminals", "");
+    mc_global.tty.setup_color_string =
+        mc_config_get_string (mc_main_config, "Colors", "base_color", "");
+    mc_global.tty.term_color_string =
+        mc_config_get_string (mc_main_config, "Colors", getenv ("TERM"), "");
+    mc_global.tty.color_terminal_string =
+        mc_config_get_string (mc_main_config, "Colors", "color_terminals", "");
 
     /* Load the directory history */
     /*    directory_history_load (); */
@@ -942,16 +936,16 @@ load_setup (void)
         buffer = mc_config_get_string (mc_main_config, "Misc", "display_codepage", "");
         if (buffer[0] != '\0')
         {
-            display_codepage = get_codepage_index (buffer);
-            cp_display = get_codepage_id (display_codepage);
+            mc_global.display_codepage = get_codepage_index (buffer);
+            cp_display = get_codepage_id (mc_global.display_codepage);
         }
         g_free (buffer);
         buffer = mc_config_get_string (mc_main_config, "Misc", "source_codepage", "");
         if (buffer[0] != '\0')
         {
             default_source_codepage = get_codepage_index (buffer);
-            source_codepage = default_source_codepage;  /* May be source_codepage doesn't need this */
-            cp_source = get_codepage_id (source_codepage);
+            mc_global.source_codepage = default_source_codepage;        /* May be source_codepage doesn't need this */
+            cp_source = get_codepage_id (mc_global.source_codepage);
         }
         g_free (buffer);
     }
@@ -960,10 +954,10 @@ load_setup (void)
     if ((autodetect_codeset[0] != '\0') && (strcmp (autodetect_codeset, "off") != 0))
         is_autodetect_codeset_enabled = TRUE;
 
-    g_free (init_translation_table (source_codepage, display_codepage));
-    buffer = (char *) get_codepage_id (display_codepage);
+    g_free (init_translation_table (mc_global.source_codepage, mc_global.display_codepage));
+    buffer = (char *) get_codepage_id (mc_global.display_codepage);
     if (buffer != NULL)
-        utf8_display = str_isutf8 (buffer);
+        mc_global.utf8_display = str_isutf8 (buffer);
 #endif /* HAVE_CHARSET */
 
     clipboard_store_path = mc_config_get_string (mc_main_config, "Misc", "clipboard_store", "");
@@ -1002,7 +996,7 @@ save_setup (gboolean save_options, gboolean save_panel_options)
 
 #ifdef HAVE_CHARSET
         mc_config_set_string (mc_main_config, "Misc", "display_codepage",
-                              get_codepage_id (display_codepage));
+                              get_codepage_id (mc_global.display_codepage));
         mc_config_set_string (mc_main_config, "Misc", "source_codepage",
                               get_codepage_id (default_source_codepage));
         mc_config_set_string (mc_main_config, "Misc", "autodetect_codeset", autodetect_codeset);
@@ -1010,7 +1004,7 @@ save_setup (gboolean save_options, gboolean save_panel_options)
         mc_config_set_string (mc_main_config, "Misc", "clipboard_store", clipboard_store_path);
         mc_config_set_string (mc_main_config, "Misc", "clipboard_paste", clipboard_paste_path);
 
-        tmp_profile = g_build_filename (home_dir, MC_USERCONF_DIR, MC_CONFIG_FILE, NULL);
+        tmp_profile = mc_config_get_full_path (MC_CONFIG_FILE);
         ret = mc_config_save_to_file (mc_main_config, tmp_profile, NULL);
         g_free (tmp_profile);
     }
@@ -1031,9 +1025,9 @@ done_setup (void)
     g_free (clipboard_paste_path);
     g_free (profile_name);
     g_free (global_profile_name);
-    g_free (color_terminal_string);
-    g_free (term_color_string);
-    g_free (setup_color_string);
+    g_free (mc_global.tty.color_terminal_string);
+    g_free (mc_global.tty.term_color_string);
+    g_free (mc_global.tty.setup_color_string);
     g_free (panels_profile_name);
     mc_config_deinit (mc_main_config);
     mc_config_deinit (mc_panels_config);
@@ -1103,7 +1097,7 @@ void
 load_key_defs (void)
 {
     /*
-     * Load keys from mc.lib before ~/.mc/ini, so that the user
+     * Load keys from mc.lib before ${XDG_CONFIG_HOME}/mc/ini, so that the user
      * definitions override global settings.
      */
     mc_config_t *mc_global_config;
@@ -1141,98 +1135,78 @@ load_anon_passwd (void)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-load_keymap_defs (void)
+load_keymap_defs (gboolean load_from_file)
 {
     /*
-     * Load keymap from GLOBAL_KEYMAP_FILE before ~/.mc/keymap, so that the user
+     * Load keymap from GLOBAL_KEYMAP_FILE before ${XDG_CONFIG_HOME}/mc/mc.keymap, so that the user
      * definitions override global settings.
      */
     mc_config_t *mc_global_keymap;
 
-    mc_global_keymap = load_setup_get_keymap_profile_config ();
+    mc_global_keymap = load_setup_get_keymap_profile_config (load_from_file);
 
     if (mc_global_keymap != NULL)
     {
+        main_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_MAIN, main_keymap, mc_global_keymap);
+        main_x_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_MAIN_EXT, main_x_keymap, mc_global_keymap);
+
+        panel_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_PANEL, panel_keymap, mc_global_keymap);
+
+        dialog_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_DIALOG, dialog_keymap, mc_global_keymap);
+
+        input_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_INPUT, input_keymap, mc_global_keymap);
+
+        listbox_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_LISTBOX, listbox_keymap, mc_global_keymap);
+
+        tree_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_TREE, tree_keymap, mc_global_keymap);
+
+        help_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
+        load_keymap_from_section (KEYMAP_SECTION_HELP, help_keymap, mc_global_keymap);
+
 #ifdef USE_INTERNAL_EDIT
         editor_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("editor", editor_keymap, mc_global_keymap);
+        load_keymap_from_section (KEYMAP_SECTION_EDITOR, editor_keymap, mc_global_keymap);
         editor_x_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("editor:xmap", editor_x_keymap, mc_global_keymap);
+        load_keymap_from_section (KEYMAP_SECTION_EDITOR_EXT, editor_x_keymap, mc_global_keymap);
 #endif
 
         viewer_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("viewer", viewer_keymap, mc_global_keymap);
+        load_keymap_from_section (KEYMAP_SECTION_VIEWER, viewer_keymap, mc_global_keymap);
         viewer_hex_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("viewer:hex", viewer_hex_keymap, mc_global_keymap);
-
-        main_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("main", main_keymap, mc_global_keymap);
-        main_x_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("main:xmap", main_x_keymap, mc_global_keymap);
-
-        panel_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("panel", panel_keymap, mc_global_keymap);
-
-        input_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("input", input_keymap, mc_global_keymap);
-
-        listbox_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("listbox", listbox_keymap, mc_global_keymap);
-
-        tree_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("tree", tree_keymap, mc_global_keymap);
-
-        help_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("help", help_keymap, mc_global_keymap);
-
-        dialog_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("dialog", dialog_keymap, mc_global_keymap);
+        load_keymap_from_section (KEYMAP_SECTION_VIEWER_HEX, viewer_hex_keymap, mc_global_keymap);
 
 #ifdef USE_DIFF_VIEW
         diff_keymap = g_array_new (TRUE, FALSE, sizeof (global_keymap_t));
-        load_keymap_from_section ("diffviewer", diff_keymap, mc_global_keymap);
+        load_keymap_from_section (KEYMAP_SECTION_DIFFVIEWER, diff_keymap, mc_global_keymap);
 #endif
+
         mc_config_deinit (mc_global_keymap);
     }
 
-    main_map = default_main_map;
-    if (main_keymap && main_keymap->len > 0)
-        main_map = (global_keymap_t *) main_keymap->data;
-
-    main_x_map = default_main_x_map;
-    if (main_x_keymap && main_x_keymap->len > 0)
-        main_x_map = (global_keymap_t *) main_x_keymap->data;
-
-    panel_map = default_panel_keymap;
-    if (panel_keymap && panel_keymap->len > 0)
-        panel_map = (global_keymap_t *) panel_keymap->data;
-
-    input_map = default_input_keymap;
-    if (input_keymap && input_keymap->len > 0)
-        input_map = (global_keymap_t *) input_keymap->data;
-
-    listbox_map = default_listbox_keymap;
-    if (listbox_keymap && listbox_keymap->len > 0)
-        listbox_map = (global_keymap_t *) listbox_keymap->data;
-
-    tree_map = default_tree_keymap;
-    if (tree_keymap && tree_keymap->len > 0)
-        tree_map = (global_keymap_t *) tree_keymap->data;
-
-    help_map = default_help_keymap;
-    if (help_keymap && help_keymap->len > 0)
-        help_map = (global_keymap_t *) help_keymap->data;
-
-    dialog_map = default_dialog_keymap;
-    if (dialog_keymap && dialog_keymap->len > 0)
-        dialog_map = (global_keymap_t *) dialog_keymap->data;
-
+    main_map = (global_keymap_t *) main_keymap->data;
+    main_x_map = (global_keymap_t *) main_x_keymap->data;
+    panel_map = (global_keymap_t *) panel_keymap->data;
+    dialog_map = (global_keymap_t *) dialog_keymap->data;
+    input_map = (global_keymap_t *) input_keymap->data;
+    listbox_map = (global_keymap_t *) listbox_keymap->data;
+    tree_map = (global_keymap_t *) tree_keymap->data;
+    help_map = (global_keymap_t *) help_keymap->data;
+#ifdef USE_INTERNAL_EDIT
+    editor_map = (global_keymap_t *) editor_keymap->data;
+    editor_x_map = (global_keymap_t *) editor_x_keymap->data;
+#endif
+    viewer_map = (global_keymap_t *) viewer_keymap->data;
+    viewer_hex_map = (global_keymap_t *) viewer_hex_keymap->data;
 #ifdef USE_DIFF_VIEW
-    diff_map = default_diff_keymap;
-    if (diff_keymap && diff_keymap->len > 0)
-        diff_map = (global_keymap_t *) diff_keymap->data;
+    diff_map = (global_keymap_t *) diff_keymap->data;
 #endif
-
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1240,6 +1214,22 @@ load_keymap_defs (void)
 void
 free_keymap_defs (void)
 {
+    if (main_keymap != NULL)
+        g_array_free (main_keymap, TRUE);
+    if (main_x_keymap != NULL)
+        g_array_free (main_x_keymap, TRUE);
+    if (panel_keymap != NULL)
+        g_array_free (panel_keymap, TRUE);
+    if (dialog_keymap != NULL)
+        g_array_free (dialog_keymap, TRUE);
+    if (input_keymap != NULL)
+        g_array_free (input_keymap, TRUE);
+    if (listbox_keymap != NULL)
+        g_array_free (listbox_keymap, TRUE);
+    if (tree_keymap != NULL)
+        g_array_free (tree_keymap, TRUE);
+    if (help_keymap != NULL)
+        g_array_free (help_keymap, TRUE);
 #ifdef USE_INTERNAL_EDIT
     if (editor_keymap != NULL)
         g_array_free (editor_keymap, TRUE);
@@ -1250,22 +1240,6 @@ free_keymap_defs (void)
         g_array_free (viewer_keymap, TRUE);
     if (viewer_hex_keymap != NULL)
         g_array_free (viewer_hex_keymap, TRUE);
-    if (main_keymap != NULL)
-        g_array_free (main_keymap, TRUE);
-    if (main_x_keymap != NULL)
-        g_array_free (main_x_keymap, TRUE);
-    if (panel_keymap != NULL)
-        g_array_free (panel_keymap, TRUE);
-    if (input_keymap != NULL)
-        g_array_free (input_keymap, TRUE);
-    if (listbox_keymap != NULL)
-        g_array_free (listbox_keymap, TRUE);
-    if (tree_keymap != NULL)
-        g_array_free (tree_keymap, TRUE);
-    if (help_keymap != NULL)
-        g_array_free (help_keymap, TRUE);
-    if (dialog_keymap != NULL)
-        g_array_free (dialog_keymap, TRUE);
 #ifdef USE_DIFF_VIEW
     if (diff_keymap != NULL)
         g_array_free (diff_keymap, TRUE);
@@ -1330,7 +1304,8 @@ panel_save_setup (struct WPanel *panel, const char *section)
     size_t i;
 
     mc_config_set_int (mc_panels_config, section, "reverse", panel->sort_info.reverse);
-    mc_config_set_int (mc_panels_config, section, "case_sensitive", panel->sort_info.case_sensitive);
+    mc_config_set_int (mc_panels_config, section, "case_sensitive",
+                       panel->sort_info.case_sensitive);
     mc_config_set_int (mc_panels_config, section, "exec_first", panel->sort_info.exec_first);
 
     mc_config_set_string (mc_panels_config, section, "sort_order", panel->sort_info.sort_field->id);
@@ -1361,29 +1336,11 @@ panel_save_setup (struct WPanel *panel, const char *section)
 void
 panels_load_options (void)
 {
-    size_t i;
-    int qmode;
-
-    /* Backward compatibility: load old parameters */
-    for (i = 0; panels_ini_options[i].opt_name != NULL; i++)
-        *panels_ini_options[i].opt_addr =
-            mc_config_get_int (mc_main_config, CONFIG_APP_SECTION,
-                               panels_ini_options[i].opt_old_name != NULL
-                               ? panels_ini_options[i].opt_old_name : panels_ini_options[i].
-                               opt_name, *panels_ini_options[i].opt_addr);
-
-    qmode = mc_config_get_int (mc_main_config, CONFIG_APP_SECTION,
-                               "quick_search_case_sensitive", (int) panels_options.qsearch_mode);
-    if (qmode < 0)
-        panels_options.qsearch_mode = QSEARCH_CASE_INSENSITIVE;
-    else if (qmode >= QSEARCH_NUM)
-        panels_options.qsearch_mode = QSEARCH_PANEL_CASE;
-    else
-        panels_options.qsearch_mode = (qsearch_mode_t) qmode;
-
-    /* overwrite by new parameters */
     if (mc_config_has_group (mc_main_config, CONFIG_PANELS_SECTION))
     {
+        size_t i;
+        int qmode;
+
         for (i = 0; panels_ini_options[i].opt_name != NULL; i++)
             *panels_ini_options[i].opt_addr =
                 mc_config_get_bool (mc_main_config, CONFIG_PANELS_SECTION,
